<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

/**
 * Class SemExpert_Oca_Model_Carrier_Oca
 * @SuppressWarnings(PHPMD.TooManyMethods)
 * @SuppressWarnings(PHPMD.CyclomaticComplexity)
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 * @SuppressWarnings(PHPMD.ExcessiveClassComplexity)
 */
class SemExpert_Oca_Model_Carrier_Oca
extends Mage_Shipping_Model_Carrier_Abstract
implements Mage_Shipping_Model_Carrier_Interface
{

    protected $_code = 'oca';
    private $_error;
    private $_validForConfirmation;

    public function isTrackingAvailable()
    {
        return true;
    }

    /**
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @param Mage_Shipping_Model_Rate_Request $request
     * @return bool|false|Mage_Core_Model_Abstract
     */
    public function collectRates(Mage_Shipping_Model_Rate_Request $request)
    {
        if (!$this->getConfigFlag('active')) {
            return false;
        }

        $this->setRequest($request);

        $volume = 0;
        $freeBoxes = 0;

        if ($request->getAllItems()) {
            foreach ($request->getAllItems() as $item) {
                /* @var $item Mage_Sales_Model_Quote_Item */
                Mage::log("Product_id: {$item->getProduct()->getId()}");
                $fullProduct = Mage::getModel('catalog/product')
                    ->load($item->getProduct()->getId());

                if ($fullProduct->getTypeId() == 'simple') {
                    $volume += $item->getQty() * floatval($fullProduct->getVolume());
                }
                Mage::log("Volume is now $volume");
                if ($item->getFreeShipping()
                    && !$item->getProduct()->isVirtual()) {

                    $freeBoxes+=$item->getQty();
                }
            }
        }
        $this->setFreeBoxes($freeBoxes);

        $rateResult = Mage::getModel('shipping/rate_result');
        if ($this->getConfigData('type') == 'O') { // per order
            $shippingPrice = $this->getConfigData('price');
        } elseif ($this->getConfigData('type') == 'I') { // per item
            $shippingPrice =
                ($request->getPackageQty() * $this->getConfigData('price'))
                - ($this->getFreeBoxes() * $this->getConfigData('price'));
        } else {
            $shippingPrice = false;
        }

        $shippingPrice = $this->getFinalPriceWithHandlingFee($shippingPrice);

        if ($shippingPrice !== false) {
            // generating two fixed shipping methods until we can use OCA WS

            $model = Mage::getModel('oca/operatory');
            $operatories = $model->getCollection()
                    ->addFieldToFilter('active', array('eq' => 1));

            // aca por ejemplo se pueden agregar filtros adicionales sobre
            // las operatorias
            Mage::dispatchEvent(
                'oca_before_load_operatories',
                array('request' => $request, 'operatories' => $operatories)
            );

            /** @ws SemExpert_Oca_Model_Ws_Ms_Adapter */
            $wsOca =  Mage::getModel('oca/ws_ms_adapter');

            /* @var $ocahelper SemExpert_Oca_Helper_Data */
            $ocahelper = Mage::helper('oca');

            $sellerParams = new Varien_Object(
                array(
                    'cuit' => $ocahelper->getCuit(),
                    'sender_zipcode' => $ocahelper->getZipcode(
                        $request->getPostcode()
                    )
                )
            );

            // hook por si un modulo custom deduce del request que tiene que
            // utilizar otro vendor
            Mage::dispatchEvent(
                'oca_set_seller_params',
                array('request' => $request, 'seller' => $sellerParams)
            );

            $cuit = $sellerParams->getCuit();
            $senderZipcode = $sellerParams->getSenderZipcode();

            Mage::log("Using as sender_zipcode: $senderZipcode");
            $receiverZipcode = $ocahelper->prepareZipcode(
                $request->getDestPostcode()
            );

            if ($cuit) {
                foreach ($operatories as $operatory) {
                    // send soap message to oca
                    $weight = floatval($request->getPackageWeight());

                    //TODO: see how to handle volume
                    Mage::log("Volumen calculado: $volume");
                    if ($volume == 0) {
                        $volume = Mage::getStoreConfig("carriers/oca/min_box_volume");
                    }
                    Mage::log("Volumen usado: $volume");

                    if ($operatory->getUsesIdci()) {
                        //$branches = $this->getBranchesFromDb($receiver_zipcode);
                        $branches = Mage::getModel('oca/branch')->getCentrosImposicionPorCP($receiverZipcode);

                        foreach ($branches as $branch) {
                            $code = $operatory->getCode() . "_" . $branch->getCode();
                            $description = $operatory->getName() . " " . $branch->getFullDescription();
                            $this->_addRate(
                                $rateResult,
                                $wsOca,
                                $operatory,
                                $weight,
                                $volume,
                                $senderZipcode,
                                $branch->getZipcode(),
                                1,
                                $cuit,
                                $code,
                                $request->getOrderSubtotal(),
                                $description
                            );
                        }
                    } else {
                        $this->_addRate(
                            $rateResult,
                            $wsOca,
                            $operatory,
                            $weight,
                            $volume,
                            $senderZipcode,
                            $receiverZipcode,
                            1,
                            $cuit,
                            $operatory->getCode(),
                            $request->getOrderSubtotal()
                        );
                    }
                }
            }
        }

        return $rateResult;
    }

    public function getTrackingInfo($tracking)
    {
        $result = $this->getTracking($tracking);

        if ($result instanceof Mage_Shipping_Model_Tracking_Result) {
            if ($trackings = $result->getAllTrackings()) {
                return $trackings[0];
            }
        } elseif (is_string($result) && !empty($result)) {
            return $result;
        }

        return false;
    }

    public function getTracking($trackings)
    {
        if (!is_array($trackings)) {
            $trackings = array($trackings);
        }

        //ups no longer support tracking for data streaming version
        //so we can only reply the popup window to ups.
        $result = Mage::getModel('shipping/tracking_result');
        foreach ($trackings as $tracking) {
            $status = Mage::getModel('shipping/tracking_result_status');
            $status->setCarrier($this->getConfigData('code'));
            $status->setCarrierTitle($this->getConfigData('title'));
            $status->setTracking($tracking);
            $status->setPopup(1);
//            $status->setUrl("http://www.oca.com.ar/Contenidos_Dinamicos/consulta_postal.asp?numero1=$tracking");
            $status->setUrl("https://www1.oca.com.ar/OEPTrackingWeb/detalleenvio.asp?numero=$tracking");
            $result->append($status);
        }

        return $result;
    }

    public function getAllowedMethods()
    {
        return array('oca'=>$this->getConfigData('name'));
    }

    /**
     * Check if carrier has shipping label option available
     *
     * @return boolean
     */
    public function isShippingLabelsAvailable()
    {
        return true;
    }

    /**
     * Return container types of carrier
     *
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param Varien_Object|null $params
     * @return array|bool
     */
    public function getContainerTypes(Varien_Object $params = null)
    {
        return array('00' => Mage::helper('oca')->__('Customer Packaging'));
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param $request
     * @return mixed
     */
    protected function getTimeForReception($request)
    {
        $diasRetiro = Mage::getStoreConfig('carriers/oca/time_for_reception');
        return $diasRetiro;
    }

    /**
     * Do request to shipment
     *
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @param Mage_Shipping_Model_Shipment_Request $request
     * @return Varien_Object
     */
    public function requestToShipment(Mage_Shipping_Model_Shipment_Request $request)
    {
        $response = new Varien_Object(array('info' => array()));

        Mage::dispatchEvent('semexpert_oca_request_to_shipment_before', array('request' => $request));
        
        $branch = "";

        $shippingMethod = $request->getShippingMethod();
        if (strpos($shippingMethod, '_')) {
            $parts = explode('_', $shippingMethod);
            $shippingMethod = array_shift($parts);
            $branch = array_shift($parts);
        }

        if ($request->getOperatoryId() && $request->getOperatoryId() != $shippingMethod) {
            $shippingMethod = $request->getOperatoryId();
            $branch = false;

            /* @var $order Mage_Sale_Model_Order */
            $order = $request->getOrderShipment()->getOrder();
            $order->addStatusHistoryComment("Enviando pedido con operatoria $shippingMethod")
                ->save();
        }

        // At this point shipping method == id operatory
        if (!$request->getCuit()) {
            $request->setCuit(Mage::helper('oca')->getCuit());
        }

        $request->setCostCenter($this->getCostCenter($request->getCuit(), $shippingMethod));

        $operatory = Mage::getModel('oca/operatory')->loadByCode($shippingMethod);

        if (!$operatory || !$operatory->getId()) {
            $this->_error = Mage::helper('oca')->__("Operatory couldn't be found: " . $shippingMethod);
            $response->setErrors($this->_error);
            return $response;
        }

        $request->setOcaOperatory($operatory);
        $request->setBranch($branch);

        /* @var $contact Mage_Admin_Model_User */
        $contact = Mage::helper('oca')->getContact();

        $request->setShipperContactPersonName($contact->getName());
        $request->setShipperEmail($contact->getEmail());

        $xmlRetiro = $this->getXmlEnvio($request);

        Mage::log($xmlRetiro);
        //$log_entry = Mage::getModel('oca/log');
        //$log_entry->setXml($xml_retiro)->save();

        // if data is not ready for confirmation we set it as to-be-confirmed despite configuration
        // we need false, not zero
        $confirmar = Mage::getStoreConfig('carriers/oca/confirm') ? $this->_validForConfirmation : false;

        $diasRetiro = Mage::getStoreConfig('carriers/oca/days_to_send');
        $franjaHoraria = $this->getTimeForReception($request);

        if (empty($diasRetiro) || empty($franjaHoraria)) {
            $confirmar = false;
        }
        
        $params = new Varien_Object(
            array(
                'usr' => Mage::getStoreConfig('carriers/oca/user'),
                'psw' => Mage::getStoreConfig('carriers/oca/password'),
                'XML_Retiro' => $xmlRetiro,
                'ConfirmarRetiro' => $confirmar,
                'DiasRetiro' => $diasRetiro,
                'FranjaHoraria' => $franjaHoraria, // 1, 2 o 3
            )
        );

        Mage::dispatchEvent('semexpert_oca_send_before', array('params' => $params));

        $wsOca =  Mage::getModel('oca/ws_ms_adapter');
        $wsdl = Mage::getStoreConfig('oca/ws/url');

        $service = 'IngresoOR';
        $result = $wsOca->exec_ingreso_or($wsdl, $service, $params->getData(), 'CodigoOperacion');

        if ($wsOca->getError()) {
            $this->_error = "Error al enviar a OCA: " . $wsOca->getError();
            Mage::log($wsOca->getError());
        }

        if (!$confirmar) {
            Mage::getSingleton('adminhtml/session')->addWarning(
                'Las etiquetas deberán ser confimadas desde el panel de e-pak'
            );
        }

        unset($params['psw']);
        Mage::log($params);
        Mage::log($result);
        //$log_entry->setParams(serialize($params))->
        //  setResult(serialize($result))->save();

        // @codingStandardsIgnoreStart
        if (isset($result->Resumen->CodigoOperacion)) {
            $trackingNumber = $this->generateTrackingNumber($request, $result);

            if (!empty($result->DetalleIngresos) &&
                !empty($result->DetalleIngresos->OrdenRetiro) &&
                !empty($result->DetalleIngresos->NumeroEnvio)
                ) {

                $shippingLabelContent = $wsOca->createLabel(
                    $wsdl,
                    $result->DetalleIngresos->OrdenRetiro,
                    $result->DetalleIngresos->NumeroEnvio
                );

            } else {
                $shippingLabelContent = $this->generatePdf($request, $result, $trackingNumber);
            }

            if ($trackingNumber) {

                $request->setMasterTrackingId($trackingNumber);
            } else {
                // this is done so that pdf is generated but tracking number won't get saved
                $trackingNumber = SemExpert_Oca_Model_Order_Shipment::DUMMY_TRACKING_NUMBER;
            }

            $data[] = array(
                'tracking_number' => $trackingNumber, //$result->getTrackingNumber(),
                'label_content'   => $shippingLabelContent    //$result->getShippingLabelContent()
            );

            $response->setInfo($data);
        } else {
            if (!$this->_error) {
                $response->setErrors(Mage::helper('oca')->__('Error informing shipping to OCA'));
            } else {
                $response->setErrors($this->_error);
            }
        }
        // @codingStandardsIgnoreEnd

        return $response;
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param $request
     * @param $result
     * @param null $trackingNumber
     * @return string
     * @throws Zend_Pdf_Exception
     */
    private function generatePdf($request, $result, $trackingNumber = null)
    {
        // Create new PDF
        $pdf = new Zend_Pdf();

        // Add new page to the document
        $page = $pdf->newPage(Zend_Pdf_Page::SIZE_A4);
        $pdf->pages[] = $page;

        // Set font
        $size = 10;
        $font = Zend_Pdf_Font::fontWithPath(Mage::getBaseDir() . '/lib/LinLibertineFont/LinLibertineC_Re-2.8.0.ttf');
        $page->setFont($font, $size);

        // Draw text
        //$page->drawText('Hello world!', 100, 510);

        /* Shipping Address */
        $order = $request->getOrderShipment()->getOrder();
        $shippingAddress = $this->_formatAddress($order->getShippingAddress()->format('pdf'));

        $this->y = 820;
        foreach ($shippingAddress as $value) {
            if ($value!=='') {
                $page->drawText(strip_tags(ltrim($value)), 35, $this->y, 'UTF-8');
                $this->y -=10;
            }

        }

        // show tracking number if available
        if ($trackingNumber) {
            $page->drawText(sprintf('Tracking Number: %s', $trackingNumber), 85, $this->y, 'UTF-8');
            $this->y -=10;
        }

        // Get PDF document as a string
        $pdfData = $pdf->render();
        return $pdfData;
    }

    protected function _formatAddress($address)
    {
        $return = array();
        foreach (explode('|', $address) as $str) {
            foreach (Mage::helper('core/string')->str_split($str, 65, true, true) as $part) {
                if (empty($part)) {
                    continue;
                }
                $return[] = $part;
            }
        }
        return $return;
    }

    /**
     * @todo obtener tracking number con
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param $request
     * @param $result
     * @return null|string
     */
    public function generateTrackingNumber($request, $result)
    {
        // @codingStandardsIgnoreStart
        if (isset($result->DetalleIngresos->NumeroEnvio)) {
            return (string) $result->DetalleIngresos->NumeroEnvio;
        }
        // @codingStandardsIgnoreEnd

        return null;
    }

    /**
     * @todo Get rid of catalog dependency. Clean in a better way i.e. entities replacement
     */
    protected function _format($text)
    {
        $text = strtr($text, array('ñ' => 'ni', 'Ñ' => 'Ni'));
        
        return preg_replace('/[\x80-\xFF]+/', '.', Mage::helper('catalog/product_url')->format($text));
    }

    /**
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @param $request
     * @return mixed
     * @throws Mage_Core_Exception
     */
    public function getXmlEnvio($request)
    {
        Mage::dispatchEvent('semexpert_oca_getxmlenvio_before', array('request' => $request));

        $packages = $request->getPackages();
        if (!is_array($packages) || !$packages) {
            Mage::throwException(Mage::helper('usa')->__('No packages for request'));
        }

        if ($request->getStoreId() != null) {
            $this->setStore($request->getStoreId());
        }

        $xmlPackages = array();

        foreach ($packages as $package) {
            $xmlPackages[] = $this->package_to_xml($package);
        }

        if ($request->getOcaAccount()) {
            $nrocuenta = $request->getOcaAccount();
        } else {
            $nrocuenta = Mage::getStoreConfig('carriers/oca/account');
        }

        /*
         *
         *
         * $request->setShipperContactPersonName($admin->getName());
        $request->setShipperContactPersonFirstName($admin->getFirstname());
        $request->setShipperContactPersonLastName($admin->getLastname());
        $request->setShipperContactCompanyName($storeInfo->getName());
        $request->setShipperContactPhoneNumber($storeInfo->getPhone());
        $request->setShipperEmail($admin->getEmail());
        $request->setShipperAddressStreet($originStreet1 . ' ' . $originStreet2);
        $request->setShipperAddressStreet1($originStreet1);
        $request->setShipperAddressStreet2($originStreet2);
        $request->setShipperAddressCity(Mage::getStoreConfig(self::XML_PATH_STORE_CITY, $shipmentStoreId));
        $request->setShipperAddressStateOrProvinceCode($shipperRegionCode);
        $request->setShipperAddressPostalCode(Mage::getStoreConfig(self::XML_PATH_STORE_ZIP, $shipmentStoreId));
        $request->setShipperAddressCountryCode(Mage::getStoreConfig(self::XML_PATH_STORE_COUNTRY_ID, $shipmentStoreId));
       */

        $this->_validForConfirmation = true;

        $direccionRetiro = $this->getDireccionRetiro($request, $packages);

        $calleRetiro = $direccionRetiro['calle_retiro'];
        $nroRetiro = $direccionRetiro['nro_retiro'];
        $cpRetiro = $direccionRetiro['cp_retiro'];
        $localidadRetiro = $direccionRetiro['localidad_retiro'];
        $provinciaRetiro = $direccionRetiro['provincia_retiro'];

        if (preg_match('/\s*[0-9][0-9]*\s*$/', $calleRetiro, $matches)) {
            $nroRetiro = $matches[0];
            $calleRetiro = substr($calleRetiro, 0, -strlen($nroRetiro));
            $nroRetiro = trim($nroRetiro);
        }

        $contactoRetiro = substr($this->_format($request->getShipperContactPersonName()), 0, 30);
        $solicitanteRetiro = substr($this->_format($request->getShipperPetitionerPersonName()), 0, 30);
        if (empty($solicitanteRetiro)) $solicitanteRetiro = $contactoRetiro;
        $emailRetiro = Mage::helper('oca')->left($request->getShipperEmail(), 100);
        $observaciones = $request->getNotes();

        $idoperativa = $request->getOcaOperatory()->getCode();

        /*
         *  $request->setRecipientContactPersonName($address->getFirstname() . ' ' . $address->getLastname());
        $request->setRecipientContactPersonFirstName($address->getFirstname());
        $request->setRecipientContactPersonLastName($address->getLastname());
        $request->setRecipientContactCompanyName($address->getCompany());
        $request->setRecipientContactPhoneNumber($address->getTelephone());
        $request->setRecipientEmail($address->getEmail());
        $request->setRecipientAddressStreet($address->getStreetFull());
        $request->setRecipientAddressStreet1($address->getStreet1());
        $request->setRecipientAddressStreet2($address->getStreet2());
        $request->setRecipientAddressCity($address->getCity());
        $request->setRecipientAddressStateOrProvinceCode($address->getRegion());
        $request->setRecipientAddressRegionCode($recipientRegionCode);
        $request->setRecipientAddressPostalCode($address->getPostcode());
        $request->setRecipientAddressCountryCode($address->getCountryId());

         */

        // obtener centro de costo
        $centrocosto = $request->getCostCenter();
        $operativa = $request->getOcaOperatory();

        if (!$operativa || !($operativa instanceof SemExpert_Oca_Model_Operatory)) {
            return "";
        }

        $idci = 0;
        if ($operativa->getUsesIdci()) {
            $idci = $request->getBranch();
        }

        if (!$request->getOrderShipment()->getId()) {
            $request->getOrderShipment()->save();
        }
        $nroRemito = $request->getOrderShipment()->getId();

        $apellidoDest = Mage::helper('oca')->left($this->_format($request->getRecipientLastName()), 30);
        if (empty($apellidoDest)) {
            $apellidoDest = Mage::helper('oca')->left(
                $this->_format($request->getRecipientContactPersonLastName()),
                30
            );
        }

        $nombreDest = Mage::helper('oca')->left(
            $this->_format($request->getRecipientFirstName()),
            30
        );

        if (empty($nombreDest)) {
            $nombreDest = Mage::helper('oca')->left(
                $this->_format($request->getRecipientContactPersonFirstName()),
                30
            );
        }

        $address = $request->getOrderShipment()
            ->getOrder()
            ->getShippingAddress();
        //$observaciones= htmlentities($this->_format($address->getComments()));

        $calleDest = Mage::helper('oca')->left(
            $this->_format($request->getRecipientAddressStreet1()),
            30
        );
        $nroDest = Mage::helper('oca')->left(
            $this->_format($request->getRecipientAddressStreet2()),
            5
        );
        //$piso_destinatario = Mage::helper('oca')->left(
        //    $this->_format($request->getRecipientAddressStreet3()),
        //    6
        //);
        //$depto_destinatario = Mage::helper('oca')->left(
        //    $this->_format($request->getRecipientAddressStreet4()),
        //    4
        //);
        $pisoDest = Mage::helper('oca')->left(
            $this->_format($address->getStreet(3)),
            6
        );
        $deptoDest = Mage::helper('oca')->left(
            $this->_format($address->getStreet(4)),
            4
        );

        $cpDestinatario = Mage::helper('oca')->prepareZipcode(
            $request->getRecipientAddressPostalCode()
        );
        $localidadDest = substr(
            $this->_format($request->getRecipientAddressCity()),
            0,
            30
        );
        $provinciaDest = Mage::helper('oca')->left(
            $request->getRecipientAddressStateOrProvinceCode(),
            30
        );
        $telefonoDest = substr(
            $request->getRecipientContactPhoneNumber(),
            0,
            30
        );
        $emailDest = substr($request->getRecipientEmail(), 0, 100);

        if (empty($apellidoDest) || empty($nombreDest) || empty($calleDest)
            || empty($nroDest) || empty($provinciaRetiro)) {
            $this->_validForConfirmation = false;
        }

        /**
         * @todo Agregar este campo porque la operatoria esta buena. Envian SMS
         */
        $celularDestinatario = $request->getRecipientContactCellNumber();

// @codingStandardsIgnoreStart
$xmlRequest = <<< XMLRequest
<?xml version="1.0"?>
    <ROWS>
    <cabecera ver="1.0" nrocuenta="$nrocuenta" />
    <retiro calle="$calleRetiro" nro="$nroRetiro" cp="$cpRetiro" localidad="$localidadRetiro" provincia="$provinciaRetiro" contacto="$contactoRetiro" email="$emailRetiro" solicitante="$solicitanteRetiro" observaciones="$observaciones" centrocosto="$centrocosto" />
    <envios>
    <envio idoperativa="$idoperativa" nroremito="$nroRemito">
    <destinatario apellido="$apellidoDest" nombre="$nombreDest" calle="$calleDest" nro="$nroDest" piso="$pisoDest" depto="$deptoDest" cp="$cpDestinatario" localidad="$localidadDest" provincia="$provinciaDest" telefono="$telefonoDest" email="$emailDest" idci="$idci" celular="$celularDestinatario" observaciones="$observaciones" />
    <paquetes>
XMLRequest;
// @codingStandardsIgnoreEnd

        foreach ($xmlPackages as $xmlPackage) {
            $xmlRequest .= $xmlPackage;
        }

$xmlRequest .= <<< XMLRequest
    </paquetes></envio></envios></ROWS>
XMLRequest;

        return preg_replace('/>\s\s*</', '><', $xmlRequest);
    }

    private function getCostCenter($cuit, $idoperativa)
    {
        $params = array(
            'CUIT' => $cuit,
            'Operativa' => $idoperativa,
        );

        $wsOca =  Mage::getModel('oca/ws_ms_adapter');
        $wsdl = Mage::getStoreConfig('oca/ws/url');

        $service = 'GetCentroCostoPorOperativa';
        $result = $wsOca->exec_soap_call(
            $wsdl,
            $service,
            $params,
            'NroCentroCosto'
        );

        // @codingStandardsIgnoreStart
        if (isset($result->NroCentroCosto)) {
            return (string) $result->NroCentroCosto;
        }
        // @codingStandardsIgnoreEnd

        return "";
    }

    private function package_to_xml($package)
    {
    /*
        $request->setPackageId($packageId);
        $request->setPackagingType($package['params']['container']);
        $request->setPackageWeight($package['params']['weight']);
        $request->setPackageParams(new Varien_Object($package['params']));
        $request->setPackageItems($package['items']);
     */
        $weightRatio = $package['params']['weight_units'] == 'POUND' ? 0.45359237 : 1;
        $dimensionRatio = $package['params']['dimension_units'] == 'INCH' ? 2.54 : 1;

        $heightCm = $dimensionRatio * $package['params']['height'];
        $lengthCm = $dimensionRatio * $package['params']['length'];
        $widthCm = $dimensionRatio * $package['params']['width'];
        $weightKg = round($weightRatio * $package['params']['weight'], 2);
        $customsValue = $package['params']['customs_value'];
        $quantity = 1;

        return  "<paquete alto=\"$heightCm\" ancho=\"$widthCm\" "
            . "largo=\"$lengthCm\" peso=\"$weightKg\" valor=\"$customsValue\" "
            . "cant=\"$quantity\" />";
    }

    protected function _fromCache($cacheKey)
    {
        $total = Mage::getSingleton('core/cache')->load($cacheKey);
        if (!$total) return false;

        $obj = new StdClass();
        // @codingStandardsIgnoreStart
        $obj->Total = $total;
        // @codingStandardsIgnoreEnd
        return $obj;
    }

    /**
     * @SuppressWarnings(PHPMD.ExcessiveParameterList)
     * @SuppressWarnings(PHPMD.UnusedFormalParameters)
     * @param $rateResult
     * @param $wsClient
     * @param $operatory
     * @param $weight
     * @param $volume
     * @param $senderZipcode
     * @param $receiverZipcode
     * @param $quantityOfPackages
     * @param $cuit
     * @param $operatoryCode
     * @param int $subtotal
     * @param bool $description
     */
    protected function _addRate(
        $rateResult,
        $wsClient,
        $operatory,
        $weight,
        $volume,
        $senderZipcode,
        $receiverZipcode,
        $quantityOfPackages,
        $cuit,
        $operatoryCode,
        $subtotal= 0,
        $description = false
    )
    {
        $calculator = $operatory->getCalculator();

        $param = new Varien_Object();
        $param->setWeight($weight);
        $param->setVolume($volume);
        $param->setSenderZipcode($senderZipcode);
        $param->setReceiverZipcode($receiverZipcode);
        $param->setQuantityOfPackages($quantityOfPackages);
        $param->setCuit($cuit);
        $param->setOperatoryCode($operatory->getCode());

        $total = $calculator->calculate($param);

        if ($total) {
            if ($operatory->getHasSecure()) {
                $total *= (1 + Mage::getStoreConfig('carriers/oca/secure_rate') / 100);
            }

            //$cache->save($result->Total, $cache_key);
            $shouldAddTax = Mage::getStoreConfig(
                'tax/calculation/shipping_includes_tax'
            );
            if ($shouldAddTax) {
                $total = 1.21 * floatval($total);
            }

            $method = Mage::getModel('shipping/rate_result_method');

            $method->setCarrier('oca');
            $method->setCarrierTitle($this->getConfigData('title'));

            $method->setMethod($operatoryCode);

            $shippingPrice = $total;

            $_freeShipping = false;
            if ($this->getRequest()->getFreeShipping() === true
                || $this->getRequest()->getPackageQty() == $this->getFreeBoxes()
            ) {
                $_freeShipping = true;
                $shippingPrice = '0.00';
            } elseif ($operatory->getPaysOnDestination()) {
                $shippingPrice = '0.00';
            }

            // in case caller needs to know
            if (!$_freeShipping) {
                $method->setRealPrice(round((string) $total, 2));
            } else {
                $method->setRealPrice($shippingPrice);
            }

            if ($description) {
                $method->setMethodTitle($description);
            } else {
                if ($operatory->getPaysOnDestination()) {
                    $methodTitle = Mage::helper('oca')->__(
                        '%s. Pay %s to courrier.',
                        $operatory->getName(),
                        Mage::helper('core')->currency(
                            $total,
                            true,
                            false
                        )
                    );
                    $method->setMethodTitle($methodTitle);
                    $method->setMethodTitlePattern(
                        Mage::helper('oca')->__(
                            '%s. Pay %%s to courrier.',
                            $operatory->getName()
                        )
                    );
                } else {
                    $method->setMethodTitle($operatory->getName());
                }
            }

            $method->setPrice($shippingPrice);
            $method->setCost($shippingPrice);

            $rateResult->append($method);
        }
    }


    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param $request
     * @param $packages
     * @return array
     */
    public function getDireccionRetiro($request, $packages)
    {
        /**
         * @todo Revisar por qué habíamos usado las siguientes 2 lineas
         */
        //$region = Mage::getModel('directory/region')
        //    ->loadByCode($request->getShipperAddressStateOrProvinceCode(),'AR');
        // $provincia_retiro = substr($region->getDefaultName(),0,30);
        $direccionRetiro = array(
            'calle_retiro' => substr($this->_format($request->getShipperAddressStreet()), 0, 30),
            'nro_retiro'    => '',
            'cp_retiro'     => Mage::helper('oca')->prepareZipcode($request->getShipperAddressPostalCode()),
            'localidad_retiro'  => substr($this->_format($request->getShipperAddressCity()), 0, 30),
            'provincia_retiro'  => substr($request->getShipperAddressStateOrProvinceCode(), 0, 30)
        );

        return $direccionRetiro;
    }

}
