<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_Model_Xml_Extractor
extends Mage_Core_Model_Abstract
{
    /**
     * Store address
     */
    const XML_PATH_STORE_ADDRESS1     = 'shipping/origin/street_line1';
    const XML_PATH_STORE_ADDRESS2     = 'shipping/origin/street_line2';
    const XML_PATH_STORE_CITY         = 'shipping/origin/city';
    const XML_PATH_STORE_REGION_ID    = 'shipping/origin/region_id';
    const XML_PATH_STORE_ZIP          = 'shipping/origin/postcode';
    const XML_PATH_STORE_COUNTRY_ID   = 'shipping/origin/country_id';

    private $_validForConfirmation;
    private $_helper;

    /**
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @param Mage_Sales_Model_Order $order
     * @return mixed
     */
    public function getXmlElocker(Mage_Sales_Model_Order $order)
    {
        $request = $this->requestFromOrder($order);

        Mage::dispatchEvent(
            'semexpert_oca_getxmlelocker_before',
            array('request' => $request)
        );

        foreach ($order->getAllItems() as $item) {
            /* @var $item Mage_Sales_Model_Order_Item */
            $this->itemToXml($item);
        }

//@todo: borrado 1
/*
        $packages = $request->getPackages();
        if (!is_array($packages) || !$packages) {
            Mage::throwException(
                Mage::helper('usa')->__('No packages for request')
            );
        }
*/
        if ($request->getStoreId() != null) {
            $this->setStore($request->getStoreId());
        }

/*
        $data = array();
        $xml_packages = array();

        foreach ($packages as $packageId => $package) {
            $xml_packages[] = $this->package_to_xml($package);
        }
*/
        
        $nrocuenta = Mage::getStoreConfig('carriers/oca/account');

        $this->_validForConfirmation = true;

        $direccionRetiro = $this->getDireccionRetiro();

        $calleRetiro = $direccionRetiro['calle_retiro'];
        $nroRetiro = $direccionRetiro['nro_retiro'];
        $cpRetiro = $direccionRetiro['cp_retiro'];
        $localidadRetiro = $direccionRetiro['localidad_retiro'];
        $provinciaRetiro = $direccionRetiro['provincia_retiro'];

        if (preg_match('/\s*[0-9][0-9]*\s*$/', $calleRetiro, $matches)) {
            $nroRetiro = $matches[0];
            $calleRetiro = substr($calleRetiro, 0, -strlen($nroRetiro));
            $nroRetiro = trim($nroRetiro);
        }

        $contactoRetiro = substr($this->helper()->textFormat($request->getShipperContactPersonName()), 0, 30);
        $solicitanteRetiro = substr($this->helper()->textFormat($request->getShipperPetitionerPersonName()), 0, 30);
        if (empty($solicitanteRetiro)) $solicitanteRetiro = $contactoRetiro;
        $emailRetiro = Mage::helper('oca')->left($request->getShipperEmail(), 100);
        $observacionesDest = $request->getNotes();

        /*
        if (!$request->getOrderShipment()->getId()) {
            $request->getOrderShipment()->save();
        } 
        $nro_remito = $request->getOrderShipment()->getId(); */
        $nroRemito = uniqid();

        $apellidoDest = Mage::helper('oca')->left($this->helper()->textFormat($request->getRecipientLastName()), 30);
        if (empty($apellidoDest)) {
            $apellidoDest = Mage::helper('oca')->left(
                $this->helper()->textFormat($request->getRecipientContactPersonLastName()),
                30
            );
        }

        $nombreDest = Mage::helper('oca')->left($this->helper()->textFormat($request->getRecipientFirstName()), 30);
        if (empty($nombreDest)) {
            $nombreDest = Mage::helper('oca')->left(
                $this->helper()->textFormat($request->getRecipientContactPersonFirstName()),
                30
            );
        }

        $address = $order->getShippingAddress();
        //$observaciones= htmlentities($this->helper()->textFormat($address->getComments()));

        $calleDest = Mage::helper('oca')->left($this->helper()->textFormat($request->getRecipientAddressStreet1()), 30);
        $nroDest = Mage::helper('oca')->left($this->helper()->textFormat($request->getRecipientAddressStreet2()), 5);
        $pisoDest = Mage::helper('oca')->left($this->helper()->textFormat($address->getStreet(3)), 6);
        $deptoDest = Mage::helper('oca')->left($this->helper()->textFormat($address->getStreet(4)), 4);

        $cpDest = Mage::helper('oca')->prepareZipcode($request->getRecipientAddressPostalCode());
        $localidadDest = substr($this->helper()->textFormat($request->getRecipientAddressCity()), 0, 30);
        $provinciaDest = Mage::helper('oca')->left($request->getRecipientAddressStateOrProvinceCode(), 30);
        $telefonoDest = substr($request->getRecipientContactPhoneNumber(), 0, 30);
        $emailDest = substr($request->getRecipientEmail(), 0, 100);

        if (empty($apellidoDest) || empty($nombreDest) || empty($calleDest)
            || empty($nroDest) || empty($provinciaRetiro)) {
            $this->_validForConfirmation = false;
        }

        /**
         * @todo Agregar este campo porque la operatoria esta buena. Envian SMS
         */
        $celularDest = $request->getRecipientContactCellNumber();

        $observacionesRetiro = '';

        //@todo: permitir configurar esto
        $idfranjahoraria = 1;
        $idciOrigen = 0; // esto solo varia para operatorias en que se lleva el paquete a la sucursal
        $fecha = date('Ymd');

        $idreserva = $order->getData('oca_elocker_reservation');
        $idci = $order->getData('oca_elocker_idci');
        $idoperativa = Mage::getStoreConfig('carriers/oca_elockers/operatory');

        $ocahelper = Mage::helper('oca');

        $cuit = $ocahelper->getCuit();
        $centrocosto = $request->getCostCenter($cuit, $idoperativa);

// @codingStandardsIgnoreStart
$xmlRequest = <<< XMLRequest
<?xml version="1.0"?>
    <ROWS>
    <cabecera ver="2.0" nrocuenta="$nrocuenta" />
	<origenes>
<origen calle="$calleRetiro" nro="$nroRetiro" piso="" depto="" cp="$cpRetiro" localidad="$localidadRetiro" provincia="$provinciaRetiro" contacto="$contactoRetiro" email="$emailRetiro"
solicitante="$solicitanteRetiro" observaciones="$observacionesRetiro" centrocosto="$centrocosto" idfranjahoraria="$idfranjahoraria" idcentroimposicionorigen="$idciOrigen" fecha="$fecha">
		<envios>
			<envio idoperativa="$idoperativa" nroremito="$nroRemito">
<destinatario apellido="$apellidoDest" nombre="$nombreDest" calle="$calleDest" nro="$nroDest" piso="$pisoDest" depto="$deptoDest" localidad="$localidadDest" provincia="$provinciaDest" cp="$cpDest" telefono="$telefonoDest" email="$emailDest" idci="$idci" idreserva="$idreserva" celular="$celularDest" observaciones="$observacionesDest"/>
			<paquetes>
XMLRequest;
// @codingStandardsIgnoreEnd

//    foreach ($xmlPackages as $xmlPackage) {
//        $xmlRequest .= $xmlPackage;
//    }


$xmlRequest .= <<< XMLRequest
				</paquetes>
			</envio>
		</envios>
		</origen>
	</origenes>
XMLRequest;

        return preg_replace('/>\s\s*</', '><', $xmlRequest);
    }

//    private function getCostCenter($cuit, $idoperativa) {
//        $params = array(
//            'CUIT' => $cuit,
//            'Operativa' => $idoperativa,
//        );
//
//        $ws =  Mage::getModel('oca/ws_ms_adapter');
//        $wsdl = Mage::getStoreConfig('oca/ws/url');
//
//        $service = 'GetCentroCostoPorOperativa';
//        $result = $ws->exec_soap_call($wsdl, $service, $params,'NroCentroCosto');
//
//        if (isset($result->NroCentroCosto))
//        {
//            return (string) $result->NroCentroCosto;
//        }
//
//        return "";
//    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param Mage_Sales_Model_Order_Item $item
     * @return string
     */
    private function itemToXml(Mage_Sales_Model_Order_Item $item)
    {
    /*
        $request->setPackageId($packageId);
        $request->setPackagingType($package['params']['container']);
        $request->setPackageWeight($package['params']['weight']);
        $request->setPackageParams(new Varien_Object($package['params']));
        $request->setPackageItems($package['items']);
     */
//        $weight_ratio = $package['params']['weight_units'] == 'POUND' ? 0.45359237 : 1;
//        $dimension_ratio = $package['params']['dimension_units'] == 'INCH' ? 2.54 : 1;

        $heightCm = 1; //$dimension_ratio * $package['params']['height'];
        $lengthCm = 1; //$dimension_ratio * $package['params']['length'];
        $widthCm = 1; //$dimension_ratio * $package['params']['width'];
        $weightKg = 0.01; //round($weight_ratio * $package['params']['weight'],2);
        $customsValue = ''; //$package['params']['customs_value'];
        $quantity = 1;

        return  "<paquete alto=\"$heightCm\" ancho=\"$widthCm\" "
            . "largo=\"$lengthCm\" peso=\"$weightKg\" valor=\"$customsValue\" cant=\"$quantity\" />";
    }

    /**
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @param Mage_Sales_Model_Order $order
     * @return Mage_Shipping_Model_Shipment_Request
     * @throws Mage_Core_Exception
     */
    private function requestFromOrder(Mage_Sales_Model_Order $order)
    {
        // setup use that would be used as contact
        $aadminId = Mage::getStoreConfig('carriers/oca/contact_id');
        $admin = Mage::getModel('admin/user')->load($aadminId);

        $address = $order->getShippingAddress();
        $shippingMethod = $order->getShippingMethod(true);
        
        $shipmentStoreId = $order->getStoreId();

        $baseCurrencyCode = Mage::app()->getStore($shipmentStoreId)->getBaseCurrencyCode();

        $shipperRegionCode = Mage::getStoreConfig(self::XML_PATH_STORE_REGION_ID, $shipmentStoreId);
        if (is_numeric($shipperRegionCode)) {
            $shipperRegionCode = Mage::getModel('directory/region')->load($shipperRegionCode)->getCode();
        }

        $recipientRegionCode = Mage::getModel('directory/region')->load($address->getRegionId())->getCode();

        $originStreet1 = Mage::getStoreConfig(self::XML_PATH_STORE_ADDRESS1, $shipmentStoreId);
        $originStreet2 = Mage::getStoreConfig(self::XML_PATH_STORE_ADDRESS2, $shipmentStoreId);
        $storeInfo = new Varien_Object(Mage::getStoreConfig('general/store_information', $shipmentStoreId));

        if (!$admin->getFirstname() || !$admin->getLastname() || !$storeInfo->getName() || !$storeInfo->getPhone()
            || !$originStreet1 || !Mage::getStoreConfig(self::XML_PATH_STORE_CITY, $shipmentStoreId)
            || !$shipperRegionCode || !Mage::getStoreConfig(self::XML_PATH_STORE_ZIP, $shipmentStoreId)
            || !Mage::getStoreConfig(self::XML_PATH_STORE_COUNTRY_ID, $shipmentStoreId)
        ) {
            Mage::throwException(
                Mage::helper('sales')->__(
                    'Insufficient information to create shipping label(s). '
                    . 'Please verify your Store Information and Shipping Settings.'
                )
            );
        }

        /** @var $request Mage_Shipping_Model_Shipment_Request */
        $request = Mage::getModel('shipping/shipment_request');
        //$request->setOrderShipment($orderShipment);
        $request->setShipperContactPersonName($admin->getName());
        $request->setShipperContactPersonFirstName($admin->getFirstname());
        $request->setShipperContactPersonLastName($admin->getLastname());
        $request->setShipperContactCompanyName($storeInfo->getName());
        $request->setShipperContactPhoneNumber($storeInfo->getPhone());
        $request->setShipperEmail($admin->getEmail());
        $request->setShipperAddressStreet(trim($originStreet1 . ' ' . $originStreet2));
        $request->setShipperAddressStreet1($originStreet1);
        $request->setShipperAddressStreet2($originStreet2);
        $request->setShipperAddressCity(Mage::getStoreConfig(self::XML_PATH_STORE_CITY, $shipmentStoreId));
        $request->setShipperAddressStateOrProvinceCode($shipperRegionCode);
        $request->setShipperAddressPostalCode(Mage::getStoreConfig(self::XML_PATH_STORE_ZIP, $shipmentStoreId));
        $request->setShipperAddressCountryCode(Mage::getStoreConfig(self::XML_PATH_STORE_COUNTRY_ID, $shipmentStoreId));
        $request->setRecipientContactPersonName(trim($address->getFirstname() . ' ' . $address->getLastname()));
        $request->setRecipientContactPersonFirstName($address->getFirstname());
        $request->setRecipientContactPersonLastName($address->getLastname());
        $request->setRecipientContactCompanyName($address->getCompany());
        $request->setRecipientContactPhoneNumber($address->getTelephone());
        $request->setRecipientEmail($address->getEmail());
        $request->setRecipientAddressStreet(trim($address->getStreet1() . ' ' . $address->getStreet2()));
        $request->setRecipientAddressStreet1($address->getStreet1());
        $request->setRecipientAddressStreet2($address->getStreet2());
        $request->setRecipientAddressCity($address->getCity());
        $request->setRecipientAddressStateOrProvinceCode($address->getRegionCode());
        $request->setRecipientAddressRegionCode($recipientRegionCode);
        $request->setRecipientAddressPostalCode($address->getPostcode());
        $request->setRecipientAddressCountryCode($address->getCountryId());
        $request->setShippingMethod($shippingMethod->getMethod());
        $request->setPackageWeight($order->getWeight());
        //$request->setPackages($orderShipment->getPackages());
        $request->setBaseCurrencyCode($baseCurrencyCode);
        $request->setStoreId($shipmentStoreId);

        return $request;
    }

    private function helper()
    {
        if (!$this->_helper) {
            $this->_helper = Mage::helper('oca/ws');
        }

        return $this->_helper;
    }
}
