<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
require_once(
    Mage::getModuleDir('controllers', 'Mage_Adminhtml'). DS . 'Sales' . DS
    . 'Order' . DS.'ShipmentController.php'
);

class SemExpert_Oca_Adminhtml_Sales_Order_ShipmentController
extends Mage_Adminhtml_Sales_Order_ShipmentController
{
    /**
     * Create shipping label for specific shipment with validation.
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @param Mage_Sales_Model_Order_Shipment $shipment
     * @return bool
     */
    protected function _createShippingLabel(Mage_Sales_Model_Order_Shipment $shipment)
    {
        if (!$shipment) {
            return false;
        }
        $carrier = $shipment->getOrder()->getShippingCarrier();
        if (!$carrier->isShippingLabelsAvailable()) {
            return false;
        }
        $shipment->setPackages($this->getRequest()->getParam('packages'));

        // all the point of this override is setting the operatory
        $operatoryId = $this->getRequest()->getParam('operatory_id');
        $operatoryCode = false;
        if ($operatoryId) {
            $operatory = Mage::getModel('oca/operatory')->load($operatoryId);
            $operatoryCode = $operatory->getCode();
        }
        $response = Mage::getModel('oca/shipping_shipping')
            ->setOperatoryId($operatoryCode)
            ->requestToShipment($shipment);

        if ($response->hasErrors()) {
            Mage::throwException($response->getErrors());
        }
        if (!$response->hasInfo()) {
            return false;
        }
        $labelsContent = array();
        $trackingNumbers = array();
        $info = $response->getInfo();
        foreach ($info as $inf) {
            if (!empty($inf['tracking_number']) && !empty($inf['label_content'])) {
                $labelsContent[] = $inf['label_content'];
                $trackingNumbers[] = $inf['tracking_number'];
            }
        }
        
        if (count($labelsContent) == 1 && stripos($labelsContent[0], '<!DOCTYPE') !== false) {
            $shipment->setShippingLabel($labelsContent[0]);
        } else {
            $outputPdf = $this->_combineLabelsPdf($labelsContent);
            $shipment->setShippingLabel($outputPdf->render());
        }
        $carrierCode = $carrier->getCarrierCode();
        $carrierTitle = Mage::getStoreConfig('carriers/'.$carrierCode.'/title', $shipment->getStoreId());
        if ($trackingNumbers) {
            foreach ($trackingNumbers as $trackingNumber) {
                $track = Mage::getModel('sales/order_shipment_track')
                        ->setNumber($trackingNumber)
                        ->setCarrierCode($carrierCode)
                        ->setTitle($carrierTitle);
                $shipment->addTrack($track);
            }
        }
        return true;
    }

}
