<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_Block_Adminhtml_Sales_Order_Shipment_Packaging
extends Mage_Adminhtml_Block_Sales_Order_Shipment_Packaging
{
    /**
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @return mixed
     * @throws Exception
     */
    public function getOperatoryId()
    {
        $orderId = $this->getRequest()->getParam('order_id');
        $shipmentId = $this->getRequest()->getParam('shipment_id');
        
        if (empty($orderId) && empty($shipmentId)) {
            return false;
        }

        if (empty($orderId)) {
            $shipment = Mage::getModel('sales/order_shipment')->load($shipmentId);
            $order = $shipment->getOrder();
        } else {
            $order = Mage::getModel('sales/order')->load($orderId);
        }
        
        if (!$order || !$order->getId()) {
            return false;
        }

        $shipmentMethod = $order->getShippingMethod();
        $parts = explode('_', $shipmentMethod);
        
        if (count($parts) < 2 || $parts[0] != 'oca') {
            return false;
        }

        $operatory = Mage::getModel('oca/operatory')->loadByCode($parts[1]);

        if (!$operatory || !$operatory->getId()) {
            return false;
        }

        return $operatory->getId();
    }

    public function getOperatories()
    {
        return Mage::getModel('oca/source_operatory')->getOptionArray();
    }

    public function getConfigDataJson()
    {
        $jsonData = parent::getConfigDataJson();
        $data = Mage::helper('core')->jsonDecode($jsonData);

        $shipmentId = $this->getShipment()->getId();
        $orderId = $this->getRequest()->getParam('order_id');
        $urlParams = array();

        $createLabelUrl = '';

        if ($shipmentId) {
            $urlParams['shipment_id'] = $shipmentId;
            $createLabelUrl = $this->getUrl('oca/adminhtml_sales_order_shipment/createLabel', $urlParams);
        } else if ($orderId) {
            $urlParams['order_id'] = $orderId;
            $createLabelUrl = $this->getUrl('oca/adminhtml_sales_order_shipment/save', $urlParams);
        }

        if (!empty($createLabelUrl)) {
            $data['createLabelUrl'] = $createLabelUrl;
        }

        return Mage::helper('core')->jsonEncode($data);
    }
}
