<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
abstract class SemExpert_Oca_Block_Operatory_Form extends Mage_Adminhtml_Block_Widget_Form
{
    /**
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    protected function _prepareForm()
    {
        $form = new Varien_Data_Form();

        $fieldset = $form->addFieldset(
            $this->getFormAction(),
            array('legend' => Mage::helper('oca')->__('Operatory Details'))
        );

        $fieldset->addField(
            'code',
            'text',
            array(
                'name'      => 'code',
                'class'     => 'validate-number',
                'title'     => Mage::helper('oca')->__('OCA Code'),
                'label'     => Mage::helper('oca')->__('OCA Code'),
                'maxlength' => '250',
                'required'  => true,
            )
        );

        $fieldset->addField(
            'name',
            'text',
            array(
                'name'      => 'name',
                'title'     => Mage::helper('oca')->__('Method Name'),
                'label'     => Mage::helper('oca')->__('Method Name'),
                'maxlength' => '250',
                'required'  => true,
            )
        );
        
        $fieldset->addField(
            'uses_idci',
            'select',
            array(
                'label'     => Mage::helper('oca')->__('Uses IDCI'),
                'name'      => 'uses_idci',
                'options'   => array(
                    0 => Mage::helper('adminhtml')->__('No'),
                    1 => Mage::helper('adminhtml')->__('Yes'),
                )
            )
        );

        $fieldset->addField(
            'has_secure',
            'select',
            array(
                'label'     => Mage::helper('oca')->__('Operatoria con seguro'),
                'name'      => 'has_secure',
                'options'   => array(
                    0 => Mage::helper('adminhtml')->__('No'),
                    1 => Mage::helper('adminhtml')->__('Yes'),
                )
            )
        );
        
        $fieldset->addField(
            'pays_on_destination',
            'select',
            array(
                'label'     => Mage::helper('oca')->__('Customer pays on destination'),
                'name'      => 'pays_on_destination',
                'options'   => array(
                    0 => Mage::helper('adminhtml')->__('No'),
                    1 => Mage::helper('adminhtml')->__('Yes'),
                )
            )
        );

        $method = $fieldset->addField(
            'calculator_method',
            'select',
            array(
                'label'     => Mage::helper('oca')->__('Método de Cálculo'),
                'title'     => Mage::helper('oca')->__('Método de Cálculo'),
                'name'      => 'calculator_method',
                'options'   => Mage::getModel('oca/source_operatory_calculator')->getOptions(),
            )
        );

        $fixedRate = $fieldset->addField(
            'fixed_rate',
            'text',
            array(
                'label'     => Mage::helper('oca')->__('Precio fijo'),
                'title'     => Mage::helper('oca')->__('Precio fijo'),
                'name'      => 'fixed_rate',
                'required'  => false,
            )
        );

        $form->setMethod('post');
        $form->setUseContainer(true);
        $form->setId('edit_form');
        $this->childFormSetup($fieldset, $form);

        $this->setForm($form);
        $this->setChild(
            'form_after',
            $this->getLayout()->createBlock('adminhtml/widget_form_element_dependence')
                ->addFieldMap($method->getHtmlId(), $method->getName())
                ->addFieldMap($fixedRate->getHtmlId(), $fixedRate->getName())
                ->addFieldDependence(
                    $fixedRate->getName(),
                    $method->getName(),
                    '1'
                )
        );
    }

    abstract public function getFormAction();
    abstract public function childFormSetup($fieldset, $form);
}
