<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_Model_Branch extends Mage_Core_Model_Abstract
{

    public function __construct( )
    {
        $this->_init('oca/branch');
    }

    public function loadByOcaId($code)
    {
        // usamos like por trailing spaces
        $collection = Mage::getModel('oca/branch')->getCollection()
            ->addFieldToFilter('code', $code);

        foreach ($collection as $operatory) {
            // por si trajimos alguno que no era
            if ($operatory->getCode() == $code) {
                return $operatory;
            }
        }

        return null;
    }

    public function loadByCodeOrNew($code)
    {
        $branch = $this->loadByOcaId($code);

        if ($branch) return $branch;

        return Mage::getModel('oca/branch')->setCode($code);
    }

    public function getFullAddress($includeLocality = false)
    {
        $address = trim($this->getAddressStreet() . " " . $this->getAddressNumber() . " " . $this->getAddressFloor());

        $locality = $this->getLocality();

        if (empty($locality) || !$includeLocality ) return $address;

        if (empty($address)) return $locality;

        return "$address - $locality";
    }

    public function getFullDescription()
    {
        return $this->getLocality() . " - " . $this->getShortName() . " - "
            . ($this->getName() ? $this->getName() : $this->getDescription())
            . " (" . $this->getFullAddress() . ")";
    }

    public function getFromOca()
    {
        return Mage::getModel('oca/ws_ms_adapter')->getCentrosImposicion();
    }

    /**
     * @see SemExpert_Oca_Model_Branch_Updater::updateFromWs
     * @deprecated Couldn't find any uses
     * @return bool
     */
    public function updateFromOca()
    {
        $branches = $this->getFromOca();

        if (!$branches) return false;

        foreach ($branches as $table) {
            // @codingStandardsIgnoreStart
            $branch = Mage::log('oca/branch')->loadByCodeOrNew($table->idCentroImposicion);

            $branch->setCode($table->idCentroImposicion);
            $branch->setShortName($table->Sigla);
            $branch->setDescription($table->Descripcion);
            $branch->setAddressStreet($table->Calle);
            $branch->setAddressNumber($table->Numero);
            $branch->setAddressFloor($table->Piso);
            $branch->setLocality($table->Localidad);
            // $branch->setZipcode();
            // @codingStandardsIgnoreEnd

            $branch->save();
        }
    }

    public function getCentrosImposicionPorCP($codigoPostal)
    {
        $wsOca = Mage::getModel('oca/ws_ms_adapter');

        $matchedBranches = array();

        try {
            $result = $wsOca->getCentrosImposicionPorCP($codigoPostal);

            // @codingStandardsIgnoreStart
            if ($result && isset($result->Table)) {
                foreach ($result->Table as $node) {
                    $matchedBranches[] = $this->processBranchNode($node);
                }
            }
            // @codingStandardsIgnoreEnd
        } catch (Exception $e) {
            Mage::logException($e);
        }

        return $matchedBranches;
    }

    function processBranchNode($node)
    {
        $branch = Mage::getModel('oca/branch')->loadByCodeOrNew($node->idCentroImposicion);

        // si no la tenemos la grabamos
        if (!$branch->getId()) {
            // @codingStandardsIgnoreStart
            $branch->setCode($node->idCentroImposicion);
            $branch->setShortName($node->Sigla);
            $branch->setDescription($node->Descripcion);
            $branch->setAddressStreet($node->Calle);
            $branch->setAddressNumber($node->Numero);
            $branch->setAddressFloor($node->Piso);
            $branch->setLocality($node->Localidad);
            // @codingStandardsIgnoreEnd
            $branch->save();
        }

        return $branch;
    }
}
