<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_Model_Branch_Updater extends Mage_Core_Model_Abstract
{
    public function updateFromWs()
    {
        $client = $this->getClient();
        $response = $client->GetCentrosImposicion(array());

        try {
            $root = null;
            foreach (array_keys((array)$response) as $key) {
                $root = $key;
                break;
            }

            if (!isset($response->$root->any)) {
                return null;
            }

            $xml = $response->$root->any; 
            $obj = simplexml_load_string($xml);

            // @codingStandardsIgnoreStart
            if (!isset($obj->NewDataSet->Table)) {
                return null;
            }

            $tables = $obj->NewDataSet->Table;
            // @codingStandardsIgnoreEnd

            $this->processTables($tables);
        } catch (Exception $e) {
            Mage::logException($e);
        }

    }

    public function getClient()
    {
        $wsdl = Mage::getStoreConfig('oca/ws/url');

        if (!$wsdl) {
            // needs explicit definition in migration 0.8.2 => 0.8.3
            $wsdl = 'http://webservice.oca.com.ar/oep_tracking/Oep_Track.asmx?wsdl';
        }

        return new SoapClient($wsdl);
    }

    private function processTables($tables)
    {
        if (empty($tables)) {
            return;
        }

        $branches = array();

        foreach ($tables as $table) {
            // @codingStandardsIgnoreStart
            $localidad = ucwords(strtolower((string) $table->Localidad));
            $branches[] = array(
                'code' => $table->idCentroImposicion,
                'short_name' => $table->Sigla,
                'description' => $table->Descripcion,
                'address_street' => $table->Calle,
                'address_number' => $table->Numero,
                'address_floor' => $table->Piso,
                'locality' => $localidad,
                'zipcode' => $table->codigopostal
            );
            // @codingStandardsIgnoreEnd
        }

        $this->getBranchResource()->updateBranches($branches);
    }

    /**
     * @return SemExpert_Oca_Model_Mysql4_Branch
     */
    public function getBranchResource()
    {
        return Mage::getModel('oca/branch')->getResource();
    }
}
