<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

/**
 * Class SemExpert_Oca_Model_Ws_Elockers
 *
 * @SuppressWarnings(PHPMD.ExcessiveClassComplexity)
 */
class SemExpert_Oca_Model_Ws_Elockers
extends Mage_Core_Model_Abstract
{
    const LOGFILE = 'elockers.log';
    const HOST = 'http://www4.oca.com.ar';
    const DEBUGGING = true;
    private $_panelsHash;

    /**
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @param $url
     * @param $data
     * @param bool $log
     * @return array|bool
     */
    private function exec_post($url, $data, $log = false)
    {
        $dataString = json_encode($data);

        if ($log) {
            $this->_log("Url: $url");
            $this->_log("Data: $dataString");
        }

        $curlHandler = curl_init($url);
        curl_setopt($curlHandler, CURLOPT_HEADER, 0);
        curl_setopt($curlHandler, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curlHandler, CURLOPT_POST, 1);
        curl_setopt($curlHandler, CURLOPT_POSTFIELDS, $dataString);
        curl_setopt(
            $curlHandler,
            CURLOPT_HTTPHEADER,
            array("Accept: application/json", "Content-Type: application/json")
        );

        $response = curl_exec($curlHandler);
        curl_close($curlHandler);

        $this->setLastJson($response);
        $resultados = json_decode($response);

        if (
            empty($resultados->response) || empty($resultados->response->status) ||
            ($resultados->response->status != 'ok') ||
            (!$this->getNoPayload() && !isset($resultados->response->data)) ) {
            $this->_log("Error llamando: $url");
            $this->_log("Se recibió: " . print_r($resultados, true));
            return $this->getNoPayload() ? false : array();
        }

        if ($log) {
            $this->_log(print_r($resultados, true));
        }

        if ($this->getNoPayload()) {
            return true;
        }

        return $resultados->response->data;
    }

    private function exec_get($url, $log = false)
    {
        $curlHandler = curl_init($url);
        curl_setopt($curlHandler, CURLOPT_HEADER, 0);
        curl_setopt($curlHandler, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curlHandler, CURLOPT_POST, 0);
        curl_setopt(
            $curlHandler,
            CURLOPT_HTTPHEADER,
            array("Accept: application/json", "Content-Type: application/x-www-form-urlencoded")
        );

        $response = curl_exec($curlHandler);
        curl_close($curlHandler);

        $this->setLastJson($response);
        $resultados = json_decode($response);

        if (
            empty($resultados->response) || empty($resultados->response->status) ||
            ($resultados->response->status != 'ok') ||
            !isset($resultados->response->data)) {
            $this->_log("Error llamando: $url");
            $this->_log("Se recibió: " . print_r($resultados, true));
            return array();
        }

        if ($log) {
            $this->_log(print_r($resultados, true));
        }

        return $resultados->response->data;
    }

    public function getPanels()
    {
        return $this->exec_get(self::HOST . '/WebApi/Api/EventosLockers.mvc/ObtenerELockers');
    }


    public function getPanelsHash()
    {
        if (!$this->_panelsHash) {
            $this->_panelsHash = array();
            foreach ($this->getPanels() as $panel) {
                // @codingStandardsIgnoreStart
                if (!empty($panel->IdCentroImposicion)) {
                    $this->_panelsHash[$panel->IdCentroImposicion] = $panel;
                }
                // @codingStandardsIgnoreEnd
            }
        }

        return $this->_panelsHash;
    }

    public function getPanelById($panelId)
    {
        $panels = $this->getPanelsHash();

        if (!isset($panels[$panelId])) {
            return false;
        }

        return $panels[$panelId];
    }

    public function solicitarRetiro(array $ordersEnvio)
    {
        try {
            $data = array(
              "userName" => Mage::getStoreConfig("carriers/oca/user"),
              "password" => Mage::getStoreConfig("carriers/oca/password"),
              "idsOrdenesRetiro" => $ordersEnvio
            );

            $this->_log('Solicitar Envio: ' . print_r($ordersEnvio, true));
            $result = $this->setNoPayload(true)
                ->exec_post(self::HOST . '/WebApi/Api/EventosLockers.mvc/SolicitarRetiro', $data, self::DEBUGGING);
            $this->_log('Resultado Solicitar Envio: ' . print_r($result, true));

            return $result;
        } catch (Exception $e) {
            Mage::logException($e);
        }

        return false;
    }

    public function getAvailableLockers($minSize = null)
    {
        $aux = $this->exec_get(self::HOST . '/WebApi/Api/EventosLockers.mvc/ObtenerDisponibilidad');
        $retval = array();

        foreach ($aux as $locker) {
            // @codingStandardsIgnoreStart
            if (!empty($minSize) && !empty($locker->id_tamanio)
                && intval($minSize) > $locker->id_tamanio) {

                // el locker es muy chico
                continue;
            }
            // @codingStandardsIgnoreEnd
            $retval[] = $locker;
        }

        return $retval;
    }

    /*
     * Paneles que tienen un elocker disponible
     *
     * Esto supone que los ids de tamaño menores están contenidos en ids
     * de tamaños mayores
     */
    public function getAvailablePanels($minSize = null)
    {
        $retval = array();
        foreach ($this->getAvailableLockers($minSize) as $locker) {
            // @codingStandardsIgnoreStart
            if (empty($locker->id_ci)) {
                continue;
            }
            // @codingStandardsIgnoreEnd

            // @codingStandardsIgnoreStart
            $panel = $this->getPanelById($locker->id_ci);
            // @codingStandardsIgnoreEnd

            if ($panel) {
                // @codingStandardsIgnoreStart
                $retval[$locker->id_ci] = Mage::getModel('oca/elocker', $this->_normalize($panel));
                // @codingStandardsIgnoreEnd
            }
        }

        return $retval;
    }

    protected function _normalize($locker)
    {
        $normalized = array();

        foreach ($locker as $key => $value) {
            // camelized to underscored
            $normalizedKey = strtolower(preg_replace('/(.)([A-Z])/', "$1_$2", $key));
            $normalized[$normalizedKey] = $value;
        }

        return $normalized;
    }

    public function getSizes()
    {
        return $this->exec_get(self::HOST . '/WebApi/Api/EventosLockers.mvc/ObtenerTamanios');
    }

    private function _log($message)
    {
        Mage::log($message, null, self::LOGFILE);
    }

    public function reserve(Mage_Sales_Model_Quote $quote, $panel)
    {
        /* @var $address Mage_Sales_Model_Quote_Address */
        $address = $quote->getShippingAddress();

        if (!$address) {
            $quoteId = $quote->getId();
            $this->_log("El quote {$quoteId} no tenía dirección de envío");
            return false;
        }

        // @codingStandardsIgnoreStart
        $reservation = array(
            "userName" => Mage::getStoreConfig("carriers/oca/user"),
            "password" => Mage::getStoreConfig("carriers/oca/password"),
            "idCentroImposicion" => $panel->IdCentroImposicion,
            "observaciones" => "",
            "nombreDest" => $address->getName(),
            "emailDest" => $address->getEmail(),
            "telefonoDest" => $address->getTelephone()
        );
        // @codingStandardsIgnoreEnd

        $sizeId = Mage::helper('oca/elockers')->getElockerSize($quote);
        $size = $this->getSize($sizeId);

        if ($size) {
            $reservation['largo'] = $size["largo"];
            $reservation['ancho'] = $size["ancho"];
            $reservation['profundidad'] = $size["profundidad"];
        } else {
            $reservation["largo"] = 1;
            $reservation["ancho"] = 1;
            $reservation["profundidad"] = 1;
        }


        $this->_log('Reservación: ' . print_r($reservation, true));
        $result = $this->exec_post(
            self::HOST . '/WebApi/Api/EventosLockers.mvc/ReservarLocker',
            $reservation,
            self::DEBUGGING
        );
        $this->_log('Resultado reservación: ' . print_r($result, true));

        if (empty($result)) {
            return false;
        }

        if (empty($result->idreserva)) {
            return false;
        }

        if (!empty($result->idci)) {
            $this->setIdci($result->idci);
        }
        return $result->idreserva;
        
        /*
http:// www4.oca.com.ar /WebApi/Api/EventosLockers.mvc/ ReservarLocker

Request
{"userName":"usuario@mail.com.ar","password":"123456","idCentroImposicion":325,
"observaciones":"","largo":10,"ancho":10,"profundidad":10,
"nombreDest":"NombreDestino","emailDest":"destino@mail.com.ar","telefonoDest":"1188336547"}

Respuesta
{"response":{"status":"ok","data":{"idreserva":"124","idci":"296"}}}
         */
    }

    public function getSize($sizeId)
    {
        $sizes = Mage::getModel('oca/catalog_product_attribute_source_size')->getSizes();
        if (!isset($sizes[$sizeId])) {
            return false;
        }

        return $sizes[$sizeId];
    }

    public function usaPanel(Mage_Sales_Model_Quote $quote)
    {
        $shippingAddress = $quote->getShippingAddress();
        if (empty($shippingAddress)) {
            return false;
        }

        return preg_match('/^oca_elockers_/', $shippingAddress->getShippingMethod());
    }
    
    public function getPanel(Mage_Sales_Model_Quote $quote)
    {
        if (!$this->usaPanel($quote)) {
            return false;
        }

        $shippingAddress = $quote->getShippingAddress();
        if (empty($shippingAddress)) {
            return false;
        }

        $parts = explode('_', $shippingAddress->getShippingMethod());

        /*
        $elockers = $this->getAvailableLockers();

        if (empty($elockers)) {
            return false;
        }

        return $elockers[0];
         *
         */
        
        $panelId = array_pop($parts);
        if (!$panelId) {
            return false;
        }

        return $this->getPanelById($panelId);
    }
}
