<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_Test_Model_Branch_Updater_AddNewBranches
extends EcomDev_PHPUnit_Test_Case
{
    /**
     * @var SemExpert_Oca_Model_Branch_Updater
     */
    private $_model;

    public function setUp()
    {
        parent::setUp();

        $this->_model = $this->getMockBuilder('SemExpert_Oca_Model_Branch_Updater')
            ->setMethods(array('getClient'))
            ->getMock();

        $soapClient = $this->getMockFromWsdl(Mage::getModuleDir('etc', 'SemExpert_Oca') . '/oca.wsdl');

        /* @codingStandardsIgnoreStart */
        $soapResponse = new stdClass();
        $soapResponse->GetCentrosImposicionResult = new stdClass();
        $soapResponse->GetCentrosImposicionResult->any =
            '<diffgr:diffgram xmlns:msdata="urn:schemas-microsoft-com:xml-msdata" '
            . 'xmlns:diffgr="urn:schemas-microsoft-com:xml-diffgram-v1">'
            . '<NewDataSet xmlns="">'
            . '<Table diffgr:id="Table1" msdata:rowOrder="0">'
                . '<idCentroImposicion>500</idCentroImposicion>'
                . '<Sigla>PB1</Sigla>'
                . '<Descripcion>Prueba Branch 1</Descripcion>'
                . '<Calle>CALLE FALSA</Calle>'
                . '<Numero>1234</Numero>'
                . '<Piso xml:space="preserve">10</Piso>'
                . '<codigopostal>5678</codigopostal>'
                . '<Localidad>BUENOS AIRES</Localidad>'
            . '</Table>'
            . '</NewDataSet></diffgr:diffgram>';
        /* @codingStandardsIgnoreEnd */

        $soapClient->method('GetCentrosImposicion')->willReturn($soapResponse);
        $this->_model->method('getClient')->willReturn($soapClient);
    }

    /**
     * @test
     * @group integration
     * @loadFixture clearBranches
     */
    public function execute()
    {
        $this->_model->updateFromWs();
        $collection = Mage::getModel('oca/branch')->getCollection();
        $this->assertCount(1, $collection);

        foreach ($collection as $branch) {
            $this->assertEquals(500, $branch->getCode());
            $this->assertEquals('PB1', $branch->getShortName());
            $this->assertEquals('Prueba Branch 1', $branch->getDescription());
            $this->assertEquals('CALLE FALSA', $branch->getAddressStreet());
            $this->assertEquals(1234, $branch->getAddressNumber());
            $this->assertEquals(10, $branch->getAddressFloor());
            $this->assertEquals(5678, $branch->getZipcode());
            $this->assertEquals('Buenos Aires', $branch->getLocality());
        }
    }
}