<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_Test_Model_Branch_Updater_UpdateExistingBranches
extends EcomDev_PHPUnit_Test_Case
{
    public function setUp()
    {
        parent::setUp();
        $this->_model = $this->getMockBuilder('SemExpert_Oca_Model_Branch_Updater')
            ->setMethods(array('getClient'))
            ->getMock();

        $soapClient = $this->getMockFromWsdl(Mage::getModuleDir('etc', 'SemExpert_Oca') . '/oca.wsdl');

        /* @codingStandardsIgnoreStart */
        $soapResponse = new stdClass();
        $soapResponse->GetCentrosImposicionResult = new stdClass();
        $soapResponse->GetCentrosImposicionResult->any =
            '<diffgr:diffgram xmlns:msdata="urn:schemas-microsoft-com:xml-msdata" '
            . 'xmlns:diffgr="urn:schemas-microsoft-com:xml-diffgram-v1">'
            . '<NewDataSet xmlns="">'
            . '<Table diffgr:id="Table1" msdata:rowOrder="0">'
            . '<idCentroImposicion>500</idCentroImposicion>'
            . '<Sigla>PB1</Sigla>'
            . '<Descripcion>Prueba Branch 1</Descripcion>'
            . '<Calle>CALLE FALSA</Calle>'
            . '<Numero>1234</Numero>'
            . '<Piso xml:space="preserve">10</Piso>'
            . '<codigopostal>5678</codigopostal>'
            . '<Localidad>BUENOS AIRES</Localidad>'
            . '</Table>'
            . '<Table diffgr:id="Table1" msdata:rowOrder="0">'
            . '<idCentroImposicion>666</idCentroImposicion>'
            . '<Sigla>PB2</Sigla>'
            . '<Descripcion>Prueba Branch 2</Descripcion>'
            . '<Calle>CALLE FALSA</Calle>'
            . '<Numero>1234</Numero>'
            . '<Piso xml:space="preserve">10</Piso>'
            . '<codigopostal>5678</codigopostal>'
            . '<Localidad>BUENOS AIRES</Localidad>'
            . '</Table>'
            . '</NewDataSet></diffgr:diffgram>';
        /* @codingStandardsIgnoreEnd */

        $soapClient->method('GetCentrosImposicion')->willReturn($soapResponse);
        $this->_model->method('getClient')->willReturn($soapClient);

    }

    /**
     * @test
     * @group integration
     * @loadFixture sampleBranch
     */
    public function execute()
    {
        $this->_model->updateFromWs();
        $collection = Mage::getModel('oca/branch')->getCollection();
        $this->assertCount(2, $collection);

        $updatedBranch = null;

        foreach ($collection as $branch) {
            if (!$updatedBranch) {
                $updatedBranch = $branch;
                break;
            }
        }

        $this->assertEquals(500, $updatedBranch->getCode());
        $this->assertEquals('PB1', $updatedBranch->getShortName());
        $this->assertEquals('Prueba Branch 1', $updatedBranch->getDescription());
        $this->assertEquals('CALLE FALSA', $updatedBranch->getAddressStreet());
        $this->assertEquals(1234, $updatedBranch->getAddressNumber());
        $this->assertEquals(10, $updatedBranch->getAddressFloor());
        $this->assertEquals(5678, $updatedBranch->getZipcode());
        $this->assertEquals('Buenos Aires', $updatedBranch->getLocality());
    }
}