<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_AdminController extends Mage_Adminhtml_Controller_Action
{
    public function labelAction()
    {
        $shipmentId = $this->getRequest()->getParam('shipment_id');
        $this->getResponse()->setBody(
            $this->getLayout()
                ->createBlock('oca/shipment_label')
                ->setShipmentId($shipmentId)
                ->toHtml()
        );
    }

    /*
    public function testAction()
    {
        die('V1.0');
    } */

    public function massSendAction()
    {
        $orderIds = $this->getRequest()->getParam('order_ids');

        if (!$this->_getOrdersToSend($orderIds)) {
            return;
        }
        
        Mage::register('order_ids', implode(',', array_values($orderIds)));

        $this->loadLayout();
        $this->_addContent($this->getLayout()->createBlock('oca/mass_send'));
        $this->renderLayout();
    }

    public function _getOrdersToSend($orderIds)
    {
        $defaultReferrer = $this->getUrl('adminhtml/sales_order');

        if (empty($orderIds)) {
            Mage::getSingleton('adminhtml/session')->addError(
                Mage::helper('oca')->__('Debe seleccionar los pedidos a enviar')
            );
            $this->getResponse()->setRedirect($defaultReferrer);
            return;
        }

        try {
            $collection = Mage::getModel('sales/order')->getCollection()
                ->addFieldToFilter('entity_id', array('entity_id' => $orderIds));

            if ($collection->count() <= 0) {
                Mage::getSingleton('adminhtml/session')->addError(
                    Mage::helper('oca')->__('Debe seleccionar los pedidos a enviar')
                );
                $this->getResponse()->setRedirect($defaultReferrer);
                return;
            }

            $cannotShip = array();

            foreach ($collection as $order) {
                if (!$order->canShip()) {
                    $cannotShip[] = $order->getIncrementId();
                }
            }

            if (!empty($cannotShip)) {
                $ids = $this->_makeList($cannotShip);
                $msg = count($cannotShip) == 1 ?
                    "El pedido $ids no puede ser enviado" :
                    "Los pedidos $ids no pueden ser enviados";
                Mage::getSingleton('adminhtml/session')->addError($msg);
                $this->_redirectReferer($defaultReferrer);
                return;
            }

            return $collection;
        } catch (Exception $e) {
            Mage::logException($e);
        }

        return false;
    }

    public function scheduleAction()
    {
        $defaultReferrer = $this->getUrl('adminhtml/sales_order');
        $orderIds = $this->getRequest()->getParam('order_ids');

        if (empty($orderIds)) {
            Mage::getSingleton('adminhtml/session')->addError(
                Mage::helper('oca')->__('Debe seleccionar los pedidos a enviar')
            );
            $this->getResponse()->setRedirect($defaultReferrer);
            return;
        }

        $operatoryId = $this->getRequest()->getParam('operatory');
        if (empty($operatoryId)) {
            Mage::getSingleton('adminhtml/session')->addError(
                Mage::helper('oca')->__('Debe seleccionar una operatoria')
            );
            $this->getResponse()->setRedirect($defaultReferrer);
            return;
        }

        $operatory = Mage::getModel('oca/operatory')->load($operatoryId);
        if (!$operatory->getCode()) {
            Mage::getSingleton('adminhtml/session')->addError(
                Mage::helper('oca')->__('Debe seleccionar una operatoria')
            );
            $this->getResponse()->setRedirect($defaultReferrer);
            return;
        }

        $collection = $this->_getOrdersToSend(explode(',', $orderIds));
        if (!$collection) {
            return;
        }

        $count = 0;
        foreach ($collection as $order) {
            $order->setUseOperatory($operatory->getCode())
                ->setSendToOca(true)
                ->save();

            $count++;
        }

        $msg = $count == 1 ? 'Se programó 1 pedido para enviar a Oca' :
            sprintf(Mage::helper('oca')->__('Se programaron %s pedidos para enviar a Oca'), $count);
        Mage::getSingleton('adminhtml/session')->addSuccess($msg);
        
        $this->getResponse()->setRedirect($defaultReferrer);
    }

    private function _makeList($array)
    {
        if (count($array) > 1) {
            $last = array_pop($array);

            return implode(', ', $array) . ' y ' . $last;
        }

        return implode(', ', $array);
    }
}
