<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_CalculatorController extends Mage_Core_Controller_Front_Action
{
    public function rateAction()
    {
        $request = $this->getRequest();
        $productId = $request->getParam('product_id');
        $receiverZipcode = Mage::helper('oca')->prepareZipcode($request->getParam('zip'));
        $product = Mage::getModel('catalog/product')->load($productId);
        $operatory = Mage::getModel('oca/operatory')->load(Mage::getStoreConfig('carriers/oca/calculator_operatory'));

        $result = new Varien_Object(array('ok' => true, 'error_message' => ''));
        Mage::dispatchEvent(
            'oca_before_calculator_rate',
            array(
                'request' => $request,
                'zipcode' => $receiverZipcode,
                'result' => $result
            )
        );
        
        if (!$result->ok) {
            $this->getResponse()->setBody(
                Zend_Json::encode(array('message' => $result->getErrorMessage(), 'status' => 'error'))
            );
            return;
        }
        
        if (!$operatory->getId()) {
            $this->getResponse()->setHttpResponseCode(503);
            $this->getResponse()->setBody('Operatoria no seleccionada');
            return;
        }
        
        /* @var $wsOca SemExpert_Oca_Model_Ws_Ms_Adapter */
        $wsOca =  Mage::getModel('oca/ws_ms_adapter');
        $wsdl = Mage::getStoreConfig('oca/ws/url');
        
        $cuit = Mage::helper('oca')->getCuit();
        $senderZipcode = Mage::helper('oca')->getZipcode();
        
        if (!$cuit) {
            $this->getResponse()->setHttpResponseCode(503);
            $this->getResponse()->setBody('CUIT pendiente de configuración');
            return;
        }
        
        $volume = Mage::getStoreConfig('carriers/oca/min_box_volume');
        $weight = floatval($product->getWeight());

        $params = array(
            'PesoTotal' => $weight, 
            'VolumenTotal' => $volume, 
            'CodigoPostalOrigen' => $senderZipcode,
            'CodigoPostalDestino' => $receiverZipcode,
            'CantidadPaquetes' => 1, 
            'Cuit' => $cuit, 
            'Operativa' => $operatory->code
        );

        Mage::log($params);
        
        $service = 'Tarifar_Envio_Corporativo';
        $result = $wsOca->exec_soap_call($wsdl, $service, $params, 'Total');

        // @codingStandardsIgnoreStart
        if (isset($result->Total)) {
            $this->getResponse()->setBody(
                Zend_Json::encode(array('total' => (float)$result->Total, 'status' => 'ok'))
            );
        } else {
            Mage::log("Fallo el cálculo de Oca");
            Mage::log(get_class($result));
            $this->getResponse()->setHttpResponseCode(503);
            $this->getResponse()
                ->setBody('Error de conexión con el calculador. Por favor vuelva a intentar en otro momento.');
        }
        // @codingStandardsIgnoreEnd

    }
}
