<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_OperatoryController extends Mage_Adminhtml_Controller_Action
{
    public function indexAction()
    {
        $this->loadLayout()
            ->_addContent($this->getLayout()->createBlock('oca/operatory_main'))
            ->renderLayout();
    }

    public function newAction()
    {
        $this->loadLayout()
            ->_addContent($this->getLayout()->createBlock('oca/operatory_new'))
            ->renderLayout();
    }

    public function postAction()
    {
        if ($data = $this->getRequest()->getPost()) {
            $operatory = Mage::getModel('oca/operatory')->setData($data);
            try {
                $operatory->save();
                
                Mage::getSingleton('adminhtml/session')->addSuccess(
                    Mage::helper('oca')->__('Operatory was successfully saved')
                );
            } catch (Exception $e){
                Mage::getSingleton('adminhtml/session')->addError($e->getMessage());
            }
        }
        $this->getResponse()->setRedirect($this->getUrl('*/*/'));
        return;
    }

    public function editAction()
    {
        $this->loadLayout();
        $this->_addContent($this->getLayout()->createBlock('oca/operatory_edit'));
        $this->renderLayout();
    }

    public function saveAction()
    {
        $operatoryId = $this->getRequest()->getParam('operatory_id', false);

        
        if ($data = $this->getRequest()->getPost()) {
            $operatory = Mage::getModel('oca/operatory')->load($operatoryId);
            $operatory->addData($data);

            try {
                $operatory->setId($operatoryId)->save();

                Mage::getSingleton('adminhtml/session')->addSuccess(
                    Mage::helper('oca')->__('Operatory successfully updated')
                );
                /*
                 * Following lines send to main grid after successful save.
                 */
                $this->getResponse()->setRedirect($this->getUrl('oca/operatory'));
                return;
            } catch (Exception $e){
                Mage::getSingleton('adminhtml/session')->addError($e->getMessage());
            }
        }
        $this->_redirectReferer();
    }


    public function deleteAction()
    {
        $operatoryId = $this->getRequest()->getParam('id', false);

        if ($this->_checkCanDelete($operatoryId)) {
            try {
                Mage::getModel('oca/operatory')->setId($operatoryId)->delete();
                Mage::getSingleton('adminhtml/session')->addSuccess(
                    Mage::helper('oca')->__('Operatory successfully deleted')
                );
                $this->getResponse()->setRedirect($this->getUrl('*/*/'));

                return;
            } catch (Exception $e){
                Mage::getSingleton('adminhtml/session')->addError($e->getMessage());
            }
        }
        $this->_redirectReferer();
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param $operatoryId
     * @return bool
     */
    private function _checkCanDelete($operatoryId)
    {
//        $operatory = Mage::getModel('oca/operatory')->load($operatoryId);

        $this->_getSession()->addError($this->__('Operatory deletion is currently disabled.'));
        return false;
    }
    
}
