<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_Helper_Config
extends Mage_Core_Helper_Abstract
{
    private $_isValid;
    private $_session;

    /*
     * For testing purposes
     */
    public function setSession($session)
    {
        $this->_session = $session;
    }

    public function getSession()
    {
        if (!$this->_session) {
            $this->_session = Mage::getSingleton('adminhtml/session');
        }

        return $this->_session;
    }

    public function validate()
    {
        $this->_isValid = true;

        $this->validateStores();

        $this->validateUsers();
    }

    protected function validateStores()
    {
        $stores = Mage::getModel('core/store')->getCollection();
        foreach ($stores as $store) {
            // skip admin
            if (!$store->getId()) {
                continue;

                $this->validateStore($store);
            }
        }
    }

    /**
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @param $store
     * @return bool
     */
    protected function validateStore($store)
    {
        $valid = true;

        $code = $store->getCode();
        $shipmentStoreId = $store->getId();

        $shipperRegionCode = Mage::getStoreConfig(self::XML_PATH_STORE_REGION_ID, $shipmentStoreId);
        if (is_numeric($shipperRegionCode)) {
            $shipperRegionCode = Mage::getModel('directory/region')->load($shipperRegionCode)->getCode();
        }

        $originStreet1 = Mage::getStoreConfig(self::XML_PATH_STORE_ADDRESS1, $shipmentStoreId);
//        $originStreet2 = Mage::getStoreConfig(self::XML_PATH_STORE_ADDRESS2, $shipmentStoreId);

        $storeInfo = new Varien_Object(Mage::getStoreConfig('general/store_information', $shipmentStoreId));

        if (!$storeInfo->getName()) {
            $this->_error($this->__("La tienda %s no tiene un nombre", $code));
            $valid = false;
        }
        
        if (!$storeInfo->getPhone()) {
            $this->_error($this->__("La tienda %s no tiene un teléfono", $code));
            $valid = false;
        }

        if (!$originStreet1) {
            $this->_error($this->__("La tienda %s no tiene cargada la dirección", $code));
            $valid = false;
        }

        if (!Mage::getStoreConfig(self::XML_PATH_STORE_CITY, $shipmentStoreId)) {
            $this->_error($this->__("La tienda %s no tiene cargada la localidad", $code));
            $valid = false;
        }

        if (!$shipperRegionCode) {
            $this->_error($this->__("La tienda %s no tiene cargada la localidad", $code));
            $valid = false;
        }
        
        if (!Mage::getStoreConfig(self::XML_PATH_STORE_ZIP, $shipmentStoreId)) {
            $this->_error($this->__("La tienda %s no tiene un código postal", $code));
            $valid = false;
        }

        if (!Mage::getStoreConfig(self::XML_PATH_STORE_COUNTRY_ID, $shipmentStoreId)) {
            $this->_error($this->__("La tienda %s no tiene cargado un país", $code));
            $valid = false;
        }

        return $valid;
    }

    protected function validateUsers()
    {
        $adminUsers = Mage::getModel('admin/user')->getCollection()
            ->addFieldToFilter('is_active', array('eq' => 1));
        foreach ($adminUsers as $admin) {
            if (!$admin->getFirstname() || !$admin->getLastname()) {
                $this->_error('No todos los administradores tienen nombre y apellido');
                return false;
            }
        }

        return true;
    }

    protected function _error($message)
    {
        $this->getSession()->addError($message);
        $this->_isValid = false;
    }

}
