<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
/**
 * @SuppressWarnings(PHPMD.ExcessiveClassComplexity)
 */
class SemExpert_Oca_Helper_Elockers extends Mage_Core_Helper_Abstract
{
    const XML_PAYMENT_METHODS = 'carriers/oca_elockers/payment_methods';
    const XML_RECEPTION_TIME = 'carriers/oca/time_for_reception';
    const NO_ITEMS = 'no-items';
    const MISSING_ITEMS = 'missing-items';
    const COMPATIBILITY_ERROR = 'campatibility';
    const MULTIPLE_PRODUCTS = 'multiple-products';
    const MULTIPLE_UNITS = 'multiple-units';
    public $lastValidationError;

    public function getElockerSizeByItems($items)
    {
        //@todo: implementar como un max

        $collection = Mage::getModel('catalog/product')->getCollection()
            ->addAttributeToFilter(
                'entity_id',
                array('in' => array_keys($items))
            )
            ->addAttributeToSelect('size_elocker_id');

        $size = null;
        foreach ($collection as $product) {
            if (empty($size)
                || intval($product->getData('size_elocker_id')) > $size) {
                $size = intval($product->getData('size_elocker_id'));
            }

        }

        return $size;
    }

    /**
     * Receives a map of productId => Qty_ordered and determines if the group is
     * compatible with eLockers
     *
     * @todo Refactor into a validator model
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @param array $items
     * @return bool
     */
    public function productsCompatibleWithElockers($items)
    {
        Mage::log($items, null, 'items-elocker.log');

        if (empty($items)) {
            // No need to query the db
            $this->lastValidationError = self::NO_ITEMS;
            return false;
        }

        /* @var Mage_Catalog_Model_Resource_Product_Collection $collection */
        $collection = Mage::getModel('catalog/product')->getCollection()
            ->addAttributeToFilter(
                'entity_id',
                array('in' => array_keys($items))
            )
            ->addAttributeToSelect('oca_elocker_compatible')
            ->addAttributeToSelect('size_elocker_id')
            ->addAttributeToSelect('oca_can_share_box');

        if ($collection->count() != count($items)) {
            $this->lastValidationError = self::MISSING_ITEMS;
            return false;
        }

        foreach ($collection as $product) {

            $qty = $items[$product->getId()];

            if (!$product->getData('oca_elocker_compatible') ||
                !$product->getData('size_elocker_id')
                ) {
                $this->lastValidationError = self::COMPATIBILITY_ERROR;
                return false;
            }

            if (count($items) > 1 && !$product->getData('oca_can_share_box')) {
                $this->lastValidationError = self::MULTIPLE_PRODUCTS;
                return false;
            }

            if (!$product->getData('oca_can_share_box') && $qty != 1) {
                $this->lastValidationError = self::MULTIPLE_UNITS;
                return false;
            }
        }

        return true;
    }

    public function getElockerSize(Mage_Sales_Model_Quote $quote)
    {
        $ids = array();

        if ($quote->getAllItems()) {
            foreach ($quote->getAllItems() as $item) {
                $ids[$item->getProductId()] = $item->getQty();
            }
        }

        return $this->getElockerSizeByItems($ids);
    }

    public function canUseElocker(Mage_Shipping_Model_Rate_Request $request)
    {
        $ids = array();

        if ($request->getAllItems()) {
            foreach ($request->getAllItems() as $item) {
                if (!$item->getHasChildren()) {
                    $ids[$item->getProductId()] = $item->getQty();
                }
            }
        }

        return $this->productsCompatibleWithElockers($ids);
    }

    public function getAvailablePaymentMethods()
    {
        $availableMethods = Mage::getStoreConfig(
            self::XML_PAYMENT_METHODS
        );

        if (empty($availableMethods)) {
            return false;
        }

        $availableMethods = trim($availableMethods);
        if (empty($availableMethods)) {
            return false;
        }

        return explode(',', $availableMethods);
    }

    public function usesElockers(Mage_Sales_Model_Quote $quote)
    {
        if (!$quote) {
            Mage::log("No quote", null, 'elockers.log');
            return false;
        }

        $shippingAddress = $quote->getShippingAddress();
        if (!$shippingAddress) {
            Mage::log("No shipping address", null, 'elockers.log');
            return false;
        }

        $shippingMethod = $shippingAddress->getShippingMethod();
        Mage::log($shippingMethod, null, 'elockers.log');

        return preg_match('/^oca_elockers/', $shippingMethod);
    }

    /**
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @param Mage_Shipping_Model_Shipment_Request $request
     * @param bool $debug
     * @return mixed
     * @throws Mage_Core_Exception
     */
    public function getXmlEnvio($request, $debug=true)
    {
        $operativa = Mage::getStoreConfig('carriers/oca_elockers/operatory');
        $nroCuenta = Mage::getStoreConfig('carriers/oca/account');
        $order = $request->getOrderShipment()->getOrder();

        $packages = $request->getPackages();
        if (!is_array($packages) || !$packages) {
            Mage::throwException(
                Mage::helper('usa')->__('No packages for request')
            );
        }

        $xmlPackages = array();

        foreach ($packages as $package) {
            $xmlPackages[] = $this->package_to_xml($package);
        }

        $direccionRetiro = $this->getDireccionRetiro($request);

        $calleRetiro = $direccionRetiro['calle_retiro'];
        $nroRetiro = $direccionRetiro['nro_retiro'];
        $cpRetiro = $direccionRetiro['cp_retiro'];
        $localidadRetiro = $direccionRetiro['localidad_retiro'];
        $provinciaRetiro = $direccionRetiro['provincia_retiro'];

        if (preg_match('/\s*[0-9][0-9]*\s*$/', $calleRetiro, $matches)) {
            $nroRetiro = $matches[0];
            $calleRetiro = substr($calleRetiro, 0, -strlen($nroRetiro));
            $nroRetiro = trim($nroRetiro);
        }

        $contactoRetiro = substr(
            $this->_format($request->getShipperContactPersonName()),
            0,
            30
        );

        $solicitanteRetiro = substr(
            $this->_format($request->getShipperPetitionerPersonName()),
            0,
            30
        );

        if (empty($solicitanteRetiro)) $solicitanteRetiro = $contactoRetiro;
        $emailRetiro = Mage::helper('oca')->left(
            $request->getShipperEmail(),
            100
        );

        $observaciones = $request->getNotes();

        $nroRemito = $order->getIncrementId();

        $apellidoDestino = Mage::helper('oca')->left(
            $this->_format($request->getRecipientLastName()),
            30
        );

        if (empty($apellidoDestino)) {
            $apellidoDestino = Mage::helper('oca')->left(
                $this->_format($request->getRecipientContactPersonLastName()),
                30
            );
        }

        $nombreDestino = Mage::helper('oca')->left(
            $this->_format($request->getRecipientFirstName()),
            30
        );

        if (empty($nombreDestino)) {
            $nombreDestino = Mage::helper('oca')->left(
                $this->_format($request->getRecipientContactPersonFirstName()),
                30
            );
        }

        if ($debug) {
            $nombreDestino = 'PRUEBA';
            $apellidoDestino = 'PRUEBA';
            $observaciones = 'PRUEBA';
        }
        
        $idci = $order->getData('oca_elocker_idci');
        $panel = Mage::getModel('oca/ws_elockers')->getPanelById($idci);

        if ($panel) {
            // @codingStandardsIgnoreStart
            $calleDestino = $panel->Calle;
            $nroDestino = $panel->Numero;
            $pisoDestino = $panel->Piso;
            // @codingStandardsIgnoreEnd
        }

        // @codingStandardsIgnoreStart
        $cpDestino = $panel->CodigoPostal;
        $localidadDestino = $panel->Localidad;
        $provinciaDestino = $panel->Provincia;
        $telefonoDestino = substr($request->getRecipientContactPhoneNumber(),0,30);
        $emailDestino = substr($request->getRecipientEmail(),0,100);
        // @codingStandardsIgnoreEnd

        /**
         * @todo Agregar este campo porque la operatoria esta buena. Envian SMS
         */
        $celularDestino = $request->getRecipientContactCellNumber();
        $fecharetiro = date('Ymd');
        $IDCIOrigen = 0;

        /**
         * @todo: extraer de la config?
         */
        $idfranjahoraria = Mage::getStoreConfig(self::XML_RECEPTION_TIME);
        $centrocosto = $this->getCostCenter(
            Mage::helper('oca')->getCuit(),
            $operativa
        );

        $idreserva = $order->getData('oca_elocker_reservation');

        /**
         * @todo Usar simpleXML u otra libreria para buildear el XML
         */
        // @codingStandardsIgnoreStart
        $xmlRequest = <<<EOF
<?xml version="1.0" encoding="ISO-8859-1"?>
<ROWS>
	<cabecera ver="2.0" nrocuenta="$nroCuenta"/>
	<origenes>
        <origen calle="$calleRetiro" nro="$nroRetiro" piso="" depto="" cp="$cpRetiro" localidad="$localidadRetiro" provincia="$provinciaRetiro" contacto="$contactoRetiro" email="$emailRetiro" solicitante="$solicitanteRetiro" observaciones="$observaciones" centrocosto="$centrocosto" idfranjahoraria="$idfranjahoraria" idcentroimposicionorigen="$IDCIOrigen" fecha="$fecharetiro" >
		<envios>
			<envio idoperativa="$operativa" nroremito="$nroRemito">
<destinatario apellido="$apellidoDestino" nombre="$nombreDestino" calle="$calleDestino" nro="$nroDestino" piso="$pisoDestino" depto="" localidad="$localidadDestino" provincia="$provinciaDestino" cp="$cpDestino" telefono="$telefonoDestino" email="$emailDestino" idci="$idci" idreserva="$idreserva" celular="$celularDestino" observaciones="Envío a eLocker OCA" />
			<paquetes>
EOF;
        // @codingStandardsIgnoreEnd

        foreach ($xmlPackages as $xmlPackage) {
            $xmlRequest .= $xmlPackage;
        }

        $xmlRequest .= <<< XMLRequest
    </paquetes></envio></envios></origen></origenes></ROWS>
XMLRequest;

        return preg_replace('/>\s\s*</', '><', $xmlRequest);
    }

    /**
     * @param Mage_Shipping_Model_Shipment_Request $request
     * @return array
     */
    public function getDireccionRetiro($request)
    {
        /**
         * @todo Revisar por qué habíamos usado las siguientes lineas
         */
        // $region = Mage::getModel('directory/region')
        //     ->loadByCode(
        //         $request->getShipperAddressStateOrProvinceCode(),
        //         'AR'
        //     );
        // $provincia_retiro = substr($region->getDefaultName(),0,30);

        $direccionRetiro = array(
            'calle_retiro' => substr(
                $this->_format($request->getShipperAddressStreet()),
                0,
                30
            ),
            'nro_retiro'    => '',
            'cp_retiro'     => Mage::helper('oca')->prepareZipcode(
                $request->getShipperAddressPostalCode()
            ),
            'localidad_retiro'  => substr(
                $this->_format($request->getShipperAddressCity()),
                0,
                30
            ),
            'provincia_retiro'  => substr(
                $request->getShipperAddressStateOrProvinceCode(),
                0,
                30
            )
        );

        return $direccionRetiro;
    }

    /**
     * @todo Get rid of catalog dependency. Clean in a better way
     * i.e. entities replacement
     */
    protected function _format($text)
    {
        $text = strtr($text, array('ñ' => 'ni', 'Ñ' => 'Ni'));
        return preg_replace(
            '/[\x80-\xFF]+/',
            '.',
            Mage::helper('catalog/product_url')->format($text)
        );
    }

    protected function _formatAddress($address)
    {
        $return = array();
        foreach (explode('|', $address) as $str) {
            $fragments = Mage::helper('core/string')
                ->str_split($str, 65, true, true);

            foreach ($fragments as $part) {
                if (empty($part)) {
                    continue;
                }
                $return[] = $part;
            }
        }
        return $return;
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param string $cuit
     * @param string $idoperativa
     * @return int
     */
    private function getCostCenter($cuit, $idoperativa)
    {

        return 1; //todo: esto no nos devuelve nada

        /*
        $params = array(
            'CUIT' => $cuit,
            'Operativa' => $idoperativa,
        );

        $ws =  Mage::getModel('oca/ws_ms_adapter');
        $wsdl = Mage::getStoreConfig('oca/ws/url');

        $service = 'GetCentroCostoPorOperativa';
        $result = $ws->exec_soap_call(
            $wsdl,
            $service,
            $params,
            'NroCentroCosto'
        );

        if (isset($result->NroCentroCosto))
        {
            return (string) $result->NroCentroCosto;
        }

        return "";
        */
    }

    private function package_to_xml($package)
    {
        /*
        $request->setPackageId($packageId);
        $request->setPackagingType($package['params']['container']);
        $request->setPackageWeight($package['params']['weight']);
        $request->setPackageParams(new Varien_Object($package['params']));
        $request->setPackageItems($package['items']);
        */
        $weightRatio = $package['params']['weight_units'] == 'POUND' ?
            0.45359237 : 1;

        $dimensionRatio = $package['params']['dimension_units'] == 'INCH' ?
            2.54 : 1;

        $heightCm = $dimensionRatio * $package['params']['height'];
        $lengthCm = $dimensionRatio * $package['params']['length'];
        $widthCm = $dimensionRatio * $package['params']['width'];
        $weightKg = round($weightRatio * $package['params']['weight'], 2);
        $customsValue = $package['params']['customs_value'];
        $quantity = 1;

        return  "<paquete alto=\"$heightCm\" ancho=\"$widthCm\" "
            . "largo=\"$lengthCm\" peso=\"$weightKg\" valor=\"$customsValue\" "
            . "cant=\"$quantity\" />";
    }
}
