<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_Model_Carrier_Elockers
extends Mage_Shipping_Model_Carrier_Abstract
implements Mage_Shipping_Model_Carrier_Interface
{
    const METHOD_FIXED = 0;
    const METHOD_OPERATORY = 1;

    const XML_PRICE_METHOD_NODE = 'carriers/oca_elockers/price_method';
    const XML_PRICE_NODE = 'carriers/oca_elockers/price_amount';
    const XML_OPERATORY_NODE = 'carriers/oca_elockers/operatory';
    const XML_TAX_SHIPPING = 'tax/calculation/shipping_includes_tax';

    const LOGFILE = 'oca_elockers.log';

    protected $_code = 'oca_elockers';
    protected $_error;

    public function isTrackingAvailable()
    {
        return true;
    }

    public function getAllowedMethods()
    {
        return array('oca'=>$this->getConfigData('name'));
    }

    /**
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @param Mage_Shipping_Model_Rate_Request $request
     * @return bool|false|Mage_Core_Model_Abstract
     */
    public function collectRates(Mage_Shipping_Model_Rate_Request $request)
    {
        if (!$this->getConfigFlag('active')) {
            return false;
        }

        /* @var $ocaHelper SemExpert_Oca_Helper_Data */
        $ocaHelper = Mage::helper('oca');

        $cuit = $ocaHelper->getCuit();
        $senderZipCode = $ocaHelper->getZipcode($request->getPostcode());

        Mage::log("Using as senderZipCode: $senderZipCode");
        $weight = floatval($request->getPackageWeight());

        //TODO: see how to handle volume
        $volume = 0;
        Mage::log("Volumen calculado: $volume");
        if ($volume == 0) {
            $volume = Mage::getStoreConfig("carriers/oca/min_box_volume");
        }
        Mage::log("Volumen usado: $volume");

        $quantityOfPackages = 1;

        if (!Mage::helper('oca/elockers')->canUseElocker($request)) {
            return false;
        }

        $rateResult = Mage::getModel('shipping/rate_result');

        $client = Mage::getModel('oca/ws_elockers');
        /* @var SemExpert_Oca_Model_Ws_Elockers $client */
        foreach ($client->getAvailablePanels() as $locker) {
            /* @var SemExpert_Oca_Model_Elocker $locker */
            if (!$locker->getCodigoPostal()) {
                continue;
            }
            $receiverZipCode = $locker->getCodigoPostal();
            $price = $this->getPrice(
                $cuit,
                $weight,
                $volume,
                $senderZipCode,
                $receiverZipCode,
                $quantityOfPackages
            );

            if (!$price) {
                continue;
            }

            $shippingPrice = $price;
            $_freeShipping = false;
            if (
                $request->getFreeShipping() === true
                || $request->getPackageQty() == $this->getFreeBoxes()

                ) {
                $_freeShipping = true;
                $shippingPrice = '0.00';
            }


            $operatoryCode = 'elockers_' . $locker->getIdCentroImposicion();
            $description = $locker->getLongDescription();

            $method = Mage::getModel('shipping/rate_result_method');

            $method->setCarrier($this->_code);
            $method->setCarrierTitle(
                Mage::getStoreConfig("carriers/{$this->_code}/title")
            );

            $method->setMethod($operatoryCode);


            // in case caller needs to know
            if (!$_freeShipping) {
                $method->setRealPrice($price);
            } else {
                $method->setRealPrice($shippingPrice);
            }

            $method->setMethodTitle($description);

            $method->setPrice($shippingPrice);
            $method->setCost($shippingPrice);

            $rateResult->append($method);
        }

        return $rateResult;
    }

    private function _log($message)
    {
        Mage::log($message, null, self::LOGFILE);
    }

    /**
     * Do request to shipment
     *
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     * @param Mage_Shipping_Model_Shipment_Request $request
     * @return Varien_Object
     */
    public function requestToShipment(
        Mage_Shipping_Model_Shipment_Request $request
    )
    {
        $response = new Varien_Object(array('info' => array()));

        /* @var $contact Mage_Admin_Model_User */
        $contact = Mage::helper('oca')->getContact();

        $request->setShipperContactPersonName($contact->getName());
        $request->setShipperEmail($contact->getEmail());

        $xmlRetiro = Mage::helper('oca/elockers')->getXmlEnvio($request);

        //echo $xml_retiro;
        //echo "\n";

        Mage::log($xmlRetiro, null, 'trace.log');
        //$log_entry = Mage::getModel('oca/log');
        //$log_entry->setXml($xml_retiro)->save();

        /**
         * If data is not ready for confirmation we set it as to-be-confirmed
         * despite configuration
         */
        $confirm = 1;

        $usr = Mage::getStoreConfig('carriers/oca/user');
        $psw = Mage::getStoreConfig('carriers/oca/password');

        $params = array(
            'usr' => $usr,
            'psw' => $psw,
            'xml_Datos' => $xmlRetiro,
            'ConfirmarRetiro' => $confirm,
            'ArchivoCliente' => '',
            'ArchivoProceso' => ''
        );


        $wsClient =  Mage::getModel('oca/ws_ms_adapter');
        $wsdl = Mage::getStoreConfig('oca/ws/url_elockers');

        $service = 'IngresoORMultiplesRetiros';
        Mage::log($params, null, 'trace.log');
        $result = $wsClient->exec_ingreso_or(
            $wsdl,
            $service,
            $params,
            'CodigoOperacion'
        );

        if ($wsClient->getError()) {
            $this->_error = "Error al enviar a OCA: " . $wsClient->getError();
            Mage::log($wsClient->getError());
        }


        unset($params['psw']);
        Mage::log($params, null, 'trace.log');
        Mage::log($result, null, 'trace.log');
        //$log_entry->setParams(serialize($params))->
        //  setResult(serialize($result))->save();

        // @codingStandardsIgnoreStart
        if (isset($result->Resumen->CodigoOperacion)) {
            // @codingStandardsIgnoreEnd
            $trackingNumber = $this->generateTrackingNumber($request, $result);

            $shippingLabelContent = '';

            // @codingStandardsIgnoreStart
            if (!empty($result->DetalleIngresos) &&
                !empty($result->DetalleIngresos->OrdenRetiro) &&
                !empty($result->DetalleIngresos->NumeroEnvio)
                ) {
            // @codingStandardsIgnoreEnd
                $shipment = $request->getOrderShipment();
                // @codingStandardsIgnoreStart
                $shipment->setData('oca_elocker_orden_retiro', (string) $result->DetalleIngresos->OrdenRetiro);
                $shipment->setData('oca_elocker_numero_envio', (string) $result->DetalleIngresos->NumeroEnvio);
                // @codingStandardsIgnoreEnd
                $shippingLabelContent = $this->generateLabel(
                    $shipment->getOcaElockerOrdenRetiro(),
                    $shipment->getOcaElockerNumeroEnvio()
                );
            }



            if ($trackingNumber) {

                $request->setMasterTrackingId($trackingNumber);
            } else {
                /**
                 * this is done so that pdf is generated but tracking number
                 * won't get saved
                 */
                $trackingNumber =
                    SemExpert_Oca_Model_Order_Shipment::DUMMY_TRACKING_NUMBER;
            }

            $data[] = array(
                'tracking_number' => $trackingNumber,
                'label_content'   => $shippingLabelContent
            );

            $response->setInfo($data);
        } else {
            if (!$this->_error) {
                $response->setErrors(
                    Mage::helper('oca')->__('Error informing shipping to OCA')
                );
            } else {
                $response->setErrors($this->_error);
            }
        }

        return $response;
    }

    public function generateLabel($orderRetiro, $numeroEnvio)
    {
        $client = Mage::getModel('oca/ws_ms_adapter');
        return $client->generateLabel($orderRetiro, $numeroEnvio);
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param $request
     * @param $result
     * @return null|string
     */
    public function generateTrackingNumber($request, $result)
    {
        // @codingStandardsIgnoreStart
        if (isset($result->DetalleIngresos->NumeroEnvio)) {
            return (string) $result->DetalleIngresos->NumeroEnvio;
        }
        // @codingStandardsIgnoreEnd

        return null;
    }

    public function getPrice($cuit, $weight, $volume, $senderZipCode,
                             $receiverZipCode, $quantityOfPackages)
    {
        $pricingMethod = Mage::getStoreConfig(self::XML_PRICE_METHOD_NODE);
        if ($pricingMethod == self::METHOD_FIXED) {
            return floatval(Mage::getStoreConfig(self::XML_PRICE_NODE));
        }

        //verificar que esté configurada la operatoria
        $operatory = Mage::getStoreConfig(self::XML_OPERATORY_NODE);

        if (empty($operatory)) {
            $this->_log(
                'No se configuró el codigo de la operatoria de e-lockers'
            );

            //verificar que se haya indicado el costo
            $price = $this->getConfigData('price');

            if (empty($price)) {
                $this->_log(
                    'No se configuró el precio de la operatoria de e-lockers'
                );
                return false;
            }

        }

        $params = array(
            'PesoTotal' => $weight,
            'VolumenTotal' => $volume,
            'CodigoPostalOrigen' => $senderZipCode, //1424,
            'CodigoPostalDestino' => $receiverZipCode,
            'CantidadPaquetes' => $quantityOfPackages,
            'Cuit' => $cuit, //'30-70932889-4',
            'Operativa' => $operatory //'65916'
        );

        $service = 'Tarifar_Envio_Corporativo';

        try {
            /* @var SemExpert_Oca_Model_Ws_Ms_Adapter $wsClient */
            $wsClient =  Mage::getModel('oca/ws_ms_adapter');

            $wsdl = Mage::getStoreConfig('oca/ws/url');

            $result = $wsClient->exec_soap_call(
                $wsdl,
                $service,
                $params,
                'Total'
            );
            $this->_log($result);

            // @codingStandardsIgnoreStart
            if (isset($result->Total)) {
                // Adjusting taxes here to minimize change
                $shouldAddTax = Mage::getStoreConfig(self::XML_TAX_SHIPPING);

                if ($shouldAddTax) {
                    $result->Total = 1.21 * floatval($result->Total);
                }

                return (string) $result->Total;
            }
            // @codingStandardsIgnoreEnd

        } catch (Exception $e) {
            Mage::logException($e);
        }
        return false;

    }
}
