<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_Model_Observer
{
    public function compatiblePaymentMethods($event)
    {
        $helper = Mage::helper('oca/elockers');

        if ($event->getQuote() && $helper->usesElockers($event->getQuote())) {
            $availableMethods = $helper->getAvailablePaymentMethods();

            $code = $event->getMethodInstance()->getCode();
            if ($availableMethods && !in_array($code, $availableMethods)) {
                $event->getResult()->isAvailable = false;
            }
            Mage::log($event->getMethodInstance()->getCode(), null, 'elockers.log');
        }
    }

    public function reserveElocker($event)
    {
        $helper = Mage::helper('oca/elockers');
        $quote = $event->getQuote();
        $order = $event->getOrder();

        if (!$helper->usesElockers($event->getQuote())) {
            return;
        }

        if (!Mage::getModel('oca/ws_elockers')->usaPanel($quote)) {
            return;
        }

        $client = Mage::getModel('oca/ws_elockers');
        try {
            $panel = $client->getPanel($quote);

            $elockerReservation = false;
            if ($panel) {
                Mage::log($panel, null, 'trace.log');
                $elockerReservation = $client->reserve($quote, $panel);
                Mage::log($elockerReservation, null, 'trace.log');
            } else {
                Mage::log('No se informo panel', null, 'trace.log');
            }

            if ($elockerReservation) {
                $order->setOcaElockerReservation($elockerReservation);
                $order->setData('oca_elocker_idci', $client->getIdci());

                return;
            }
        } catch (exception $e) {
            Mage::logException($e);
        }

        // si llegó hasta acá, tenía que haber reservado pero no lo logró.

        // rehabilito el carrito
        $event->getQuote()->setIsActive(true)->save();

        // Y lanzo excepcion para que no pueda avanzar
        Mage::throwException(
            "No se pudo reserver el e-locker. Por favor espere unos instantes "
            . "o seleccione otro método de envío"
        );
    }

    public function sendToElocker($event)
    {
        try {

            $invoice = $event->getInvoice();
            /* @var $order Mage_Sales_Model_Order */
            $order = $invoice->getOrder(); //$event->getOrder();

            if ($order->getOcaElockerIdci()) {
                $sender = Mage::getModel('oca/elocker_sender');
                $sender->shipById($order->getId());
            }

            $ordersToConfirm = array();

            /* @var $shipment Mage_Sales_Model_Order_Shipment */
            foreach ($order->getShipmentsCollection() as $shipment) {
                if ($shipment->getOcaElockerOrdenRetiro()) {
                    $ordersToConfirm[] = $shipment->getOcaElockerOrdenRetiro();
                }
            }

            if ($ordersToConfirm) {
                $wsOca = Mage::getModel('oca/ws_elockers');
                $wsOca->solicitarRetiro($ordersToConfirm);
            } else {
                Mage::log('No se SolicitaRetiro', null, 'elockers.log');
            }
        } catch (Exception $ex) {
            Mage::logException($ex);
        }
    }
}
