<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
/**
 * Test class for SemExpert_Oca_Model_Carrier_Oca.
 * Generated by PHPUnit on 2012-09-21 at 16:37:02.
 * @SuppressWarnings(PHPMD.TooManyMethods)
 */
class SemExpert_Oca_Test_Model_Carrier_Oca extends EcomDev_PHPUnit_Test_Case
{
    const DEFAULT_MOCK_RATE = 100;
    const DEFAULT_MOCK_BRANCH = 42; // Matches fixture

    /**
     * @var SemExpert_Oca_Model_Carrier_Oca
     */
    protected $_object;

    /**
     * Sets up the fixture, for example, opens a network connection.
     * This method is called before a test is executed.
     */
    protected function setUp()
    {
        $this->_object = Mage::getModel('oca/carrier_oca');
        $this->_mockWs();
    }

    /**
     * @covers SemExpert_Oca_Model_Carrier_Oca::isTrackingAvailable
     */
    public function testIsTrackingAvailable()
    {
        $this->assertTrue($this->_object->isTrackingAvailable());
    }

    /**
     * @test
     * @group integration
     * @dataProvider dataProvider
     * @loadFixture
     */
    public function testCollectRates($data)
    {
        $this->mockSession('customer/session', array('renewSession'));
        $request = Mage::getModel('shipping/rate_request');
        /* @var $request Mage_Shipping_Model_Rate_Request */
        $request->setData($data['request']);
        /* @var $quote Mage_Sales_Model_Quote */
        $quote = Mage::getModel('sales/quote')->loadByIdWithoutStore($data['quote_id']);
        $allItems = array();

        foreach ($quote->getItemsCollection() as $item) {
            $allItems[] = $item;
        }

        $request->setAllItems($allItems);
        $result = $this->_object->collectRates($request);
        $this->assertInstanceOf('Mage_Shipping_Model_Rate_Result', $result);
        $rates = $result->getAllRates();
        $this->assertCount(2, $rates);

        foreach ($rates as $rate) {
            $this->assertEquals(self::DEFAULT_MOCK_RATE, $rate->getPrice());
        }
    }

    /**
     * @covers SemExpert_Oca_Model_Carrier_Oca::collectRates
     * @loadFixture
     */
    public function testCollectRatesOcaDisabled()
    {
        $request = Mage::getModel('shipping/rate_request');
        $result = $this->_object->collectRates($request);
        $this->assertFalse($result);
    }

    /**
     * @covers SemExpert_Oca_Model_Carrier_Oca::collectRates
     * @todo Implement testCollectRates().
     */
    public function testCollectRatesWithDifferentBaseCurrency()
    {
        // Remove the following lines when you implement this test.
        $this->markTestIncomplete(
            'This test has not been implemented yet.'
        );
    }

    /**
     * @covers SemExpert_Oca_Model_Carrier_Oca::collectRates
     * @todo Implement testCollectRates().
     */
    public function testCollectRatesWithDifferentBaseCurrencyAndNoConversionRate()
    {
        // Remove the following lines when you implement this test.
        $this->markTestIncomplete(
            'This test has not been implemented yet.'
        );
    }

    /**
     * @covers SemExpert_Oca_Model_Carrier_Oca::getTrackingInfo
     * @todo Implement testGetTrackingInfo().
     */
    public function testGetTrackingInfo()
    {
        // Remove the following lines when you implement this test.
        $this->markTestIncomplete(
            'This test has not been implemented yet.'
        );
    }

    /**
     * @covers SemExpert_Oca_Model_Carrier_Oca::getTracking
     * @todo Implement testGetTracking().
     */
    public function testGetTracking()
    {
        // Remove the following lines when you implement this test.
        $this->markTestIncomplete(
            'This test has not been implemented yet.'
        );
    }

    /**
     * @covers SemExpert_Oca_Model_Carrier_Oca::getAllowedMethods
     * @todo Implement testGetAllowedMethods().
     */
    public function testGetAllowedMethods()
    {
        // Remove the following lines when you implement this test.
        $this->markTestIncomplete(
            'This test has not been implemented yet.'
        );
    }

    /**
     * @covers SemExpert_Oca_Model_Carrier_Oca::isShippingLabelsAvailable
     */
    public function testIsShippingLabelsAvailable()
    {
        $this->assertTrue($this->_object->isShippingLabelsAvailable());
    }

    /**
     * @covers SemExpert_Oca_Model_Carrier_Oca::getContainerTypes
     * @todo Implement testGetContainerTypes().
     */
    public function testGetContainerTypes()
    {
        // Remove the following lines when you implement this test.
        $this->markTestIncomplete(
            'This test has not been implemented yet.'
        );
    }

    /**
     * @covers SemExpert_Oca_Model_Carrier_Oca::requestToShipment
     * @todo Implement testRequestToShipment().
     */
    public function testRequestToShipment()
    {
        // Remove the following lines when you implement this test.
        $this->markTestIncomplete(
            'This test has not been implemented yet.'
        );
    }

    /**
     * @covers SemExpert_Oca_Model_Carrier_Oca::generateTrackingNumber
     * @todo Implement testGenerateTrackingNumber().
     */
    public function testGenerateTrackingNumber()
    {
        // Remove the following lines when you implement this test.
        $this->markTestIncomplete(
            'This test has not been implemented yet.'
        );
    }

    /**
     * @test
     * @group integration
     * @todo Create a new shipment for testing
     * @dataProvider dataProvider
     * @loadFixture
     */
    public function testGetXmlEnvioSinIdci($data)
    {
        /* @var $request Mage_Shipping_Model_Shipment_Request */
        $request = Mage::getModel('shipping/shipment_request');
        $request->setData($data['request']);

        /* @var $operatory SemExpert_Oca_Model_Operatory */
        $operatory = Mage::getModel('oca/operatory');
        $operatory->setData($data['operatory']);

        $shippingAddress = Mage::getModel('sales/order_address');
        $shippingAddress->setData($data['shipping_address']);

        $order = Mage::getModel('sales/order');
        $order->setShippingAddress($shippingAddress);

        $orderShipment = Mage::getModel('sales/order_shipment');
        $orderShipment->setId("numero remito");
        $orderShipment->setOrder($order);

        $request->setOcaOperatory($operatory);
        $request->setOrderShipment($orderShipment);
        $request->setPackages($data['packages']);

        $xml = $this->_object->getXmlEnvio($request);

$expected = <<<XMLEnvio
<?xml version="1.0"?>
<ROWS>
	<cabecera ver="1.0" nrocuenta="numero cuenta OCA"/>
	<retiro
			calle="Calle retiro"
			nro="1234"
			cp="6666"
			localidad="localidad retiro"
			provincia="provincia retiro"
			contacto="contacto retiro"
			email="e-mail del contacto retiro"
			solicitante="solicitante retiro"
			observaciones="observaciones retiro"
			centrocosto="13"
	/>
	<envios>
		<envio idoperativa="12358" nroremito="numero remito"	>
			<destinatario
					apellido="Apellido destinatario"
					nombre="Nombre destinatario"
					calle="Calle destinatario"
					nro="1624"
					piso="1"
					depto="D"
					cp="1617"
					localidad="localidad destinatario"
					provincia="provincia destinatario"
					telefono="telefono destinatario"
					email="email destinatario"
					idci="0"
					celular="celular destinatario"
        			observaciones="observaciones retiro"
			/>
			<paquetes>
				<paquete alto="10" ancho="11" largo="12" peso="15" valor="200" cant="1" />
				<paquete alto="11" ancho="20" largo="20" peso="30" valor="300" cant="1" />
			</paquetes>
		</envio>
	</envios>
</ROWS>
XMLEnvio;

        $this->assertXmlStringEqualsXmlString($expected, $xml);

        // Test escaped strings
        $request->setRecipientContactPersonLastName("Apellido déstíñátarió");
        $request->setRecipientContactPersonFirstName("Ñombre déstínátarió");
        $xml = $this->_object->getXmlEnvio($request);

        $simpleXml = new SimpleXMLElement($xml);

        $destinatario = $simpleXml->xpath('/ROWS/envios/envio/destinatario');

        $this->assertEquals('Apellido destiniatario', $destinatario[0]['apellido']);
        $this->assertEquals('Niombre destinatario', $destinatario[0]['nombre']);
    }

    /**
     * @test
     * @group integration
     * @todo Create a new shipment for testing
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    public function testGetXmlEnvioConIdci()
    {
        /* @var $request Mage_Shipping_Model_Shipment_Request */
        $request = Mage::getModel('shipping/shipment_request');

        /* @var $operatory SemExpert_Oca_Model_Operatory */
        $operatory = Mage::getModel('oca/operatory');
        $operatory->setCode('12358');
        $operatory->setUsesIdci(true);

        Mage::app()->getStore()->setConfig('carriers/oca/account', 'numero cuenta OCA');

        $shippingAddress = Mage::getModel('sales/order_address');
        $shippingAddress->setStreet(array("Calle destinatario","1624","1","D"));

        $order = Mage::getModel('sales/order');
        $order->setShippingAddress($shippingAddress);

        $orderShipment = Mage::getModel('sales/order_shipment');
        $orderShipment->setId("numero remito");
        $orderShipment->setOrder($order);

        $request->setOcaOperatory($operatory);
        $request->setOrderShipment($orderShipment);
        $request->setBranch('777'); // idci

        // <retiro>
        $request->setShipperAddressStreet("Calle retiro\n1234");
        $request->setShipperAddressPostalCode("6666");
        $request->setShipperAddressCity("localidad retiro");
        $request->setShipperAddressStateOrProvinceCode("provincia retiro");
        $request->setShipperContactPersonName("contacto retiro");
        $request->setShipperEmail("e-mail del contacto retiro");
        $request->setShipperPetitionerPersonName("solicitante retiro");
        $request->setNotes("observaciones retiro");
        $request->setCostCenter("13");
        // </retiro>

        // <envios>
        // <envio>
        // <destinatario>
        $request->setRecipientContactPersonLastName("Apellido destinatario");
        $request->setRecipientContactPersonFirstName("Nombre destinatario");
        $request->setRecipientAddressStreet1("Calle destinatario");
        $request->setRecipientAddressStreet2("1624");
        $request->setRecipientAddressStreet3("1");
        $request->setRecipientAddressStreet4("D");
        $request->setRecipientAddressPostalCode("1617");
        $request->setRecipientAddressCity("localidad destinatario");
        $request->setRecipientAddressStateOrProvinceCode("provincia destinatario");
        $request->setRecipientContactPhoneNumber("telefono destinatario");
        $request->setRecipientEmail("email destinatario");
        $request->setRecipientContactCellNumber("celular destinatario");
        // </destinatario>
        // <paquetes>
        $packages = array(
            array('params' => array(
                'weight_units' => 'KILOGRAM',
                'dimension_units' => 'CENTIMETER',
                'height' => '10',
                'width' => '11',
                'length' => '12',
                'weight' => '15',
                'customs_value' => '200'
            )),
            array('params' => array(
                'weight_units' => 'KILOGRAM',
                'dimension_units' => 'CENTIMETER',
                'height' => '11',
                'width' => '20',
                'length' => '20',
                'weight' => '30',
                'customs_value' => '300'
            )),
        );
        $request->setPackages($packages);
        // </packages>

        $xml = $this->_object->getXmlEnvio($request);

$expected = <<<XMLEnvio
<?xml version="1.0"?>
<ROWS>
	<cabecera ver="1.0" nrocuenta="numero cuenta OCA"/>
	<retiro
			calle="Calle retiro"
			nro="1234"
			cp="6666"
			localidad="localidad retiro"
			provincia="provincia retiro"
			contacto="contacto retiro"
			email="e-mail del contacto retiro"
			solicitante="solicitante retiro"
			observaciones="observaciones retiro"
			centrocosto="13"
	/>
	<envios>
		<envio idoperativa="12358" nroremito="numero remito"	>
			<destinatario
					apellido="Apellido destinatario"
					nombre="Nombre destinatario"
					calle="Calle destinatario"
					nro="1624"
					piso="1"
					depto="D"
					cp="1617"
					localidad="localidad destinatario"
					provincia="provincia destinatario"
					telefono="telefono destinatario"
					email="email destinatario"
					idci="777"
					celular="celular destinatario"
        			observaciones="observaciones retiro"
			/>
			<paquetes>
				<paquete alto="10" ancho="11" largo="12" peso="15" valor="200" cant="1" />
				<paquete alto="11" ancho="20" largo="20" peso="30" valor="300" cant="1" />
			</paquetes>
		</envio>
	</envios>
</ROWS>
XMLEnvio;

        $this->assertXmlStringEqualsXmlString($expected, $xml);

        // Test escaped strings
        $request->setRecipientContactPersonLastName("Apellido déstíñátarió");
        $request->setRecipientContactPersonFirstName("Ñombre déstínátarió");
        $xml = $this->_object->getXmlEnvio($request);

        $simpleXml = new SimpleXMLElement($xml);

        $destinatario = $simpleXml->xpath('/ROWS/envios/envio/destinatario');

        $this->assertEquals('Apellido destiniatario', $destinatario[0]['apellido']);
        $this->assertEquals('Niombre destinatario', $destinatario[0]['nombre']);
    }

    private function _mockWs()
    {
        $wsClient = $this->getModelMock('oca/ws_ms_adapter', array('exec_soap_call', 'getCentrosImposicionPorCP'));

        $wsClient->expects($this->any())
            ->method('exec_soap_call')
            ->will(
                $this->returnCallback(
                    function() {
                        $result = new SimpleXMLElement(
                            '<result>'
                            . '<Tarifador>15</Tarifador>'
                            . '<Precio>' . self::DEFAULT_MOCK_RATE . '</Precio>'
                            . '<idTiposervicio>1</idTiposervicio>'
                            . '<Ambito>Local</Ambito>'
                            . '<PlazoEntrega>3</PlazoEntrega>'
                            . '<Adicional>0.0000</Adicional>'
                            . '<Total>' . self::DEFAULT_MOCK_RATE . '</Total>'
                            . '<XML>&lt;row Tarifador="15" Precio="' . self::DEFAULT_MOCK_RATE . '"/&gt;</XML>'
                            . '</result>'
                        );
                        return $result;
                    }
                )
            );

        $wsClient->expects($this->any())
            ->method('getCentrosImposicionPorCP')
            ->will(
                $this->returnCallback(
                    function() {
                        $result = new SimpleXMLElement(
                            '<NewDataSet><Table>
                                <idCentroImposicion>' . self::DEFAULT_MOCK_BRANCH . '</idCentroImposicion>
                                <Sigla>VSF</Sigla>
                                <Descripcion>VELEZ SARFIELD CI             </Descripcion>
                                <Calle>AV. VELEZ SARFIELD            </Calle>
                                <Numero>1860 </Numero>
                                <Piso xml:space="preserve">     </Piso>
                            </Table></NewDataSet>'
                        );
                        return $result;
                    }
                )
            );

        $this->replaceByMock('model', 'oca/ws_ms_adapter', $wsClient);
    }

}
