<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

class SemExpert_Oca_Test_Model_Carrier_Oca_CalculoConSeguro
extends EcomDev_PHPUnit_Test_Case
{
    /**
     * @var ReflectionMethod
     */
    private $_method;

    /**
     * @var SemExpert_Oca_Model_Carrier_Oca
     */
    private $_model;
    private $_operatoryMock;

    public function setUp()
    {
        parent::setUp();

        $class = new ReflectionClass('SemExpert_Oca_Model_Carrier_Oca');
        $this->_method = $class->getMethod('_addRate');
        $this->_method->setAccessible(true);

        $this->_model = $this->getMOckBuilder('SemExpert_Oca_Model_Carrier_Oca')
            ->setMethods(array('getConfigData'))
            ->getMock();

        $this->_model->method('getConfigData')
            ->will(
                $this->returnCallback(
                    function ($setting)
                    {
                        if ($setting == 'secure_rate') {
                            return 10;
                        }

                        return null;
                    }
                )
            );

        /**
         * Mock del calculador que siempre devuelve $100
         */
        $calculatorMock = $this->getMockBuilder('SemExpert_Oca_Model_Rate_Calculator_Fixed')
            ->disableOriginalConstructor()
            ->setMethods(array('calculate'))
            ->getMock();

        $calculatorMock->expects($this->once())->method('calculate')->willReturn(100);

        $this->_operatoryMock = $this->getMockBuilder('SemExpert_Oca_Model_Operatory')
            ->disableOriginalConstructor()
            ->setMethods(array('getCalculator'))
            ->getMock();

        $this->_operatoryMock->method('getCalculator')->willReturn($calculatorMock);
    }

    /**
     * El calculo del envio es 100 (por el mock), el seguro es 10% y el subtotal
     * del pedido es $500 por lo que el costo final del envío debería ser $150
     *
     *
     * @test
     * @covers SemExpert_Oca_Model_Carrier_Oca::_addRate
     * @uses SemExpert_Oca_Model_Operatory
     * @group unit
     * @ticket 7582
     */
    public function execute()
    {
        $rateResult = Mage::getModel('shipping/rate_result');
        $rateRequest = new Mage_Shipping_Model_Rate_Request;
        $rateRequest->setFreeShipping(false);
        $rateRequest->setPackageQty(1);

        $this->_model->setRequest($rateRequest);

        $this->_operatoryMock->setHasSecure(true);

        $this->_method->invoke(
            $this->_model,
            $rateResult,
            $this->_operatoryMock,
            100,
            0.1,
            1428,
            1606,
            1,
            '30-12345678-1',
            1234,
            500
        );

        $rates = $rateResult->getRatesByCarrier('oca');

        $this->assertCount(1, $rates);
        $this->assertEquals(150, $rates[0]['price']);

    }
}