<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_BranchController extends Mage_Adminhtml_Controller_Action
{
    public function loadAction()
    {
        try {
            $updater = Mage::getModel('oca/branch_updater');
            /* @var $updater SemExpert_Oca_Model_Branch_Updater */
            $updater->updateFromWs();
            Mage::getSingleton('adminhtml/session')->addSuccess(
                Mage::helper('oca')->__('Branches successfuly updated')
            );
        } catch (Exception $e){
            Mage::getSingleton('adminhtml/session')->addError(
                Mage::helper('oca')->__('An error occurred while updating branches from Oca')
            );
        }
        $this->getResponse()->setRedirect($this->getUrl('oca/branch'));
    }
    
    public function indexAction()
    {
        $this->loadLayout()
            ->_addContent($this->getLayout()->createBlock('oca/branch_main'))
            ->renderLayout();
    }

    public function newAction()
    {
        $this->loadLayout()
            ->_addContent($this->getLayout()->createBlock('oca/branch_new'))
            ->renderLayout();
    }

    public function postAction()
    {
        if ($data = $this->getRequest()->getPost()) {
            $branch = Mage::getModel('oca/branch')->setData($data);
            try {
                $branch->save();
                Mage::getSingleton('adminhtml/session')->addSuccess(
                    Mage::helper('oca')->__('Branch was successfully saved')
                );
            } catch (Exception $e){
                Mage::getSingleton('adminhtml/session')->addError($e->getMessage());
            }
        }
        $this->getResponse()->setRedirect($this->getUrl('*/*/'));
        return;
    }

    public function editAction()
    {
        $this->loadLayout();
        $this->_addContent($this->getLayout()->createBlock('oca/branch_edit'));
        $this->renderLayout();
    }

    public function saveAction()
    {
        $branchId = $this->getRequest()->getParam('branch_id', false);

        
        if ($data = $this->getRequest()->getPost()) {
            $branch = Mage::getModel('oca/branch')->load($branchId);
            $branch->addData($data);

            try {
                $branch->setId($branchId)->save();

                Mage::getSingleton('adminhtml/session')->addSuccess(
                    Mage::helper('oca')->__('Branch successfully updated')
                );
                /*
                 * Following lines send to main grid after successful save.
                 */
                $this->getResponse()->setRedirect($this->getUrl('oca/branch'));
                return;
            } catch (Exception $e){
                Mage::getSingleton('adminhtml/session')->addError($e->getMessage());
            }
        }
        $this->_redirectReferer();
    }


    public function deleteAction()
    {
        $branchId = $this->getRequest()->getParam('id', false);

        if ($this->_checkCanDelete($branchId)) {
            try {
                Mage::getModel('oca/branch')->setId($branchId)->delete();
                Mage::getSingleton('adminhtml/session')->addSuccess(
                    Mage::helper('oca')->__('Branch successfully deleted')
                );
                $this->getResponse()->setRedirect($this->getUrl('*/*/'));

                return;
            } catch (Exception $e){
                Mage::getSingleton('adminhtml/session')->addError($e->getMessage());
            }
        }
        $this->_redirectReferer();
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @param $branchId
     * @return bool
     */
    private function _checkCanDelete($branchId)
    {
//        $branch = Mage::getModel('oca/branch')->load($branchId);

        $this->_getSession()->addError($this->__('Branch deletion is currently disabled.'));
        return false;
    }

}
