<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_Helper_Data extends Mage_Core_Helper_Abstract
{
    public function getCuit()
    {
        $cuit = Mage::getStoreConfig('shipping/tax/cuit');
        
        // sanity check to avoid errors in ws call
        if (!$cuit) return false;

        $cuit = str_replace('-', '', $cuit);
        
        if (strlen($cuit) != 11) return false;
        
        if (preg_match("/[^0-9]/", $cuit)) {
            return false;
        }
        
        return substr($cuit, 0, 2) . '-' . substr($cuit, 2, 8) . '-' . substr($cuit, 10);
    }
    
    public function getRequestTotalInclTax($request)
    {
        $retval = 0;
        foreach ($request->getAllItems() as $item) {
            $retval += $item->getRowTotalInclTax();
        }

        Mage::log("Total is $retval");
        return $retval;
    }

    public function left($str, $size)
    {
        if (!$str) return '';
        
        return substr($str, 0, $size);
    }
    
    public function getZipcode($zipcode = false)
    {
        if (!$zipcode || empty($zipcode)) return false;
        
        return $this->prepareZipcode($zipcode);
    }
    
    /**
     * Make sure that zipcode is exactly 4 chars long with only decimal chars
     * or els Oca will die with an exception
     * 
     * @assert ('C1234') == '1234'
     * @assert ('1234X') == '1234'
     * @assert ('12Z34') == '1234'
     * @assert ('C1234X') == '1234'
     * @assert ('C12Z34') == '1234'
     * @assert ('12Z34X') == '1234'
     * @assert ('123456') == '1234'
     * @assert('123') == '0123'
     * @param type $zipcode
     * @return type 
     */
    public function prepareZipcode($zipcode)
    {
        // Remove non-numeric chars
        $zipcode = preg_replace('/[^0-9]/', '', $zipcode);
        
        $len = strlen($zipcode);
        
        if ($len < 4) {
            // Pad to 4 chars
            $zipcode = str_pad($zipcode, 4, "0", STR_PAD_LEFT);
        }
        
        if ($len > 4) {
            // Trim to 4 chars
            $zipcode = substr($zipcode, 0, 4);
        }
        
        return $zipcode;
    }
    
    public function getContact()
    {
        $collection = Mage::getModel('admin/user')->getCollection();
        
        $contactId = Mage::getStoreConfig('carriers/oca/contact_id');
        if ($contactId) {
            $collection->addFieldToFilter('user_id', $contactId);
        }
        
        if ($collection->count() > 0) {
            foreach ($collection as $item) {
                return $item;
            }
        }
        
        return Mage::getModel('admin/user');
    }
}
