<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_Model_Branch_Updater extends Mage_Core_Model_Abstract
{
    public function updateFromWs()
    {
        $wsdl = Mage::getStoreConfig('oca/ws/url');
        
        // precisa definicion explicita en migracion 0.8.2 => 0.8.3
        if (!$wsdl) $wsdl = 'http://webservice.oca.com.ar/oep_tracking/Oep_Track.asmx?wsdl';

        $client = new SoapClient($wsdl);
        $response = $client->GetCentrosImposicion(); 

        try {
            $root = null;
            foreach (array_keys($response) as $key) {
                $root = $key;
                break;
            }

            if (!isset($response->$root->any)) {
                return null;
            }

            $xml = $response->$root->any; 
            $obj = simplexml_load_string($xml);

            // @codingStandardsIgnoreStart
            if (!isset($obj->NewDataSet->Table)) {
                return null;
            }

            $tables = $obj->NewDataSet->Table;
            // @codingStandardsIgnoreEnd

            $this->processTables($tables);
        } catch (Exception $e) {
            Mage::logException($e);
        }

    }

    private function processTables($tables)
    {
        foreach ($tables as $table) {
            // @codingStandardsIgnoreStart
            $localidad = ucwords(strtolower((string) $table->Localidad));

            $branch = Mage::getModel('oca/branch')->loadByCodeOrNew($table->idCentroImposicion);
            $branch->setCode($table->idCentroImposicion);
            $branch->setShortName($table->Sigla);
            $branch->setDescription($table->Descripcion);
            $branch->setAddressStreet($table->Calle);
            $branch->setAddressNumber($table->Numero);
            $branch->setAddressFloor($table->Piso);
            $branch->setLocality($localidad);
            // @codingStandardsIgnoreEnd
            // $branch->setZipcode();
            $branch->save();

        }
    }
}
