<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_Model_Shipping_Shipping
extends Mage_Shipping_Model_Shipping
{
    private $_operatoryId;

    /*
     * Mage_Shipping_Model_Shipping don't extends from a class implementing
     * the getter and setter magic methods, so I have to implement them by
     * hand
     */
    public function setOperatoryId($operatoryId)
    {
        $this->_operatoryId = $operatoryId;
        return $this;
    }

    public function getOperatoryId()
    {
        return $this->_operatoryId;
    }

    //@todo: this is overkilling, deberia encontrar una manera menos intrusiva
    // de cambiar la operatoria
    /**
     * Prepare and do request to shipment
     *
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @param Mage_Sales_Model_Order_Shipment $orderShipment
     * @return Varien_Object
     */
    public function requestToShipment(Mage_Sales_Model_Order_Shipment $orderShipment)
    {
        $admin = Mage::getSingleton('admin/session')->getUser();
        $order = $orderShipment->getOrder();
        $address = $order->getShippingAddress();
        $shippingMethod = $order->getShippingMethod(true);
        $shipmentStoreId = $orderShipment->getStoreId();
        $shipmentCarrier = $order->getShippingCarrier();
        $baseCurrencyCode = Mage::app()->getStore($shipmentStoreId)->getBaseCurrencyCode();
        if (!$shipmentCarrier) {
            Mage::throwException('Invalid carrier: ' . $shippingMethod->getCarrierCode());
        }
        $shipperRegionCode = Mage::getStoreConfig(self::XML_PATH_STORE_REGION_ID, $shipmentStoreId);
        if (is_numeric($shipperRegionCode)) {
            $shipperRegionCode = Mage::getModel('directory/region')->load($shipperRegionCode)->getCode();
        }

        $recipientRegionCode = Mage::getModel('directory/region')->load($address->getRegionId())->getCode();

        $originStreet1 = Mage::getStoreConfig(self::XML_PATH_STORE_ADDRESS1, $shipmentStoreId);
        $originStreet2 = Mage::getStoreConfig(self::XML_PATH_STORE_ADDRESS2, $shipmentStoreId);
        $storeInfo = new Varien_Object(Mage::getStoreConfig('general/store_information', $shipmentStoreId));

        if (!$admin->getFirstname()) {
            Mage::throwException(
                Mage::helper('oca')->__('No se puede crear el envío: Falta el nombre del contacto')
            );
        }

        if (!$admin->getLastname()) {
            Mage::throwException(
                Mage::helper('oca')->__('No se puede crear el envío: Falta el apellido del contacto')
            );
        }

        if (!$storeInfo->getName()) {
            Mage::throwException(
                Mage::helper('oca')->__('No se puede crear el envío: Falta el nombre de la tienda')
            );
        }

        if (!$storeInfo->getPhone()) {
            Mage::throwException(
                Mage::helper('oca')->__('No se puede crear el envío: Falta el teléfono de la tienda')
            );
        }

        if (!$originStreet1) {
            Mage::throwException(
                Mage::helper('oca')->__('No se puede crear el envío: Falta la dirección de la tienda')
            );
        }

        if (!Mage::getStoreConfig(self::XML_PATH_STORE_CITY, $shipmentStoreId)) {
            Mage::throwException(
                Mage::helper('oca')->__('No se puede crear el envío: Falta la localidad de la tienda')
            );
        }

        if (!$shipperRegionCode) {
            Mage::throwException(
                Mage::helper('oca')->__('No se puede crear el envío: Falta la provincia de la tienda')
            );
        }

        if (!Mage::getStoreConfig(self::XML_PATH_STORE_ZIP, $shipmentStoreId)) {
            Mage::throwException(
                Mage::helper('oca')->__('No se puede crear el envío: Falta el código postal de la tienda')
            );
        }

        if (!Mage::getStoreConfig(self::XML_PATH_STORE_COUNTRY_ID, $shipmentStoreId)) {
            Mage::throwException(
                Mage::helper('oca')->__('No se puede crear el envío: Falta el país de la tienda')
            );
        }

        if (!$admin->getFirstname() || !$admin->getLastname() || !$storeInfo->getName() || !$storeInfo->getPhone()
            || !$originStreet1 || !Mage::getStoreConfig(self::XML_PATH_STORE_CITY, $shipmentStoreId)
            || !$shipperRegionCode || !Mage::getStoreConfig(self::XML_PATH_STORE_ZIP, $shipmentStoreId)
            || !Mage::getStoreConfig(self::XML_PATH_STORE_COUNTRY_ID, $shipmentStoreId)
        ) {
            Mage::throwException(
                Mage::helper('sales')->__(
                    'Insufficient information to create shipping label(s). '
                    . 'Please verify your Store Information and Shipping Settings.'
                )
            );
        }

        /** @var $request Mage_Shipping_Model_Shipment_Request */
        $request = Mage::getModel('shipping/shipment_request');
        $request->setOrderShipment($orderShipment);
        $request->setShipperContactPersonName($admin->getName());
        $request->setShipperContactPersonFirstName($admin->getFirstname());
        $request->setShipperContactPersonLastName($admin->getLastname());
        $request->setShipperContactCompanyName($storeInfo->getName());
        $request->setShipperContactPhoneNumber($storeInfo->getPhone());
        $request->setShipperEmail($admin->getEmail());
        $request->setShipperAddressStreet(trim($originStreet1 . ' ' . $originStreet2));
        $request->setShipperAddressStreet1($originStreet1);
        $request->setShipperAddressStreet2($originStreet2);
        $request->setShipperAddressCity(Mage::getStoreConfig(self::XML_PATH_STORE_CITY, $shipmentStoreId));
        $request->setShipperAddressStateOrProvinceCode($shipperRegionCode);
        $request->setShipperAddressPostalCode(Mage::getStoreConfig(self::XML_PATH_STORE_ZIP, $shipmentStoreId));
        $request->setShipperAddressCountryCode(Mage::getStoreConfig(self::XML_PATH_STORE_COUNTRY_ID, $shipmentStoreId));
        $request->setRecipientContactPersonName(trim($address->getFirstname() . ' ' . $address->getLastname()));
        $request->setRecipientContactPersonFirstName($address->getFirstname());
        $request->setRecipientContactPersonLastName($address->getLastname());
        $request->setRecipientContactCompanyName($address->getCompany());
        $request->setRecipientContactPhoneNumber($address->getTelephone());
        $request->setRecipientEmail($address->getEmail());
        $request->setRecipientAddressStreet(trim($address->getStreet1() . ' ' . $address->getStreet2()));
        $request->setRecipientAddressStreet1($address->getStreet1());
        $request->setRecipientAddressStreet2($address->getStreet2());
        $request->setRecipientAddressCity($address->getCity());
        $request->setRecipientAddressStateOrProvinceCode($address->getRegionCode());
        $request->setRecipientAddressRegionCode($recipientRegionCode);
        $request->setRecipientAddressPostalCode($address->getPostcode());
        $request->setRecipientAddressCountryCode($address->getCountryId());
        $request->setShippingMethod($shippingMethod->getMethod());
        $request->setPackageWeight($order->getWeight());
        $request->setPackages($orderShipment->getPackages());
        $request->setBaseCurrencyCode($baseCurrencyCode);
        $request->setStoreId($shipmentStoreId);

        if ($this->getOperatoryId()) {
            $request->setOperatoryId($this->getOperatoryId());
        }

        return $shipmentCarrier->requestToShipment($request);
    }

}
