<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_Model_Ws_Ms_Adapter extends Mage_Core_Model_Abstract
{
    /**
     *
     * @param type $wsdl
     * @param type $method
     * @param type $params
     * @param type $field Para verificar llamada exitosa
     * @return type 
     */
    public function exec_soap_call($wsdl, $method, $params, $field)
    {
        $client = new SoapClient($wsdl);
        $response = $client->$method($params);

        Mage::log($response, null, 'exec_soap_call.log', true);

        $table = null;
        
        try {
            $root = null;
            foreach (array_keys($response) as $key) {
                $root = $key;
                break;
            }
            
            if (!isset($response->$root->any)) {
                return null;
            }

            $xml = $response->$root->any; 
            $obj = simplexml_load_string($xml);

            // @codingStandardsIgnoreStart
            if (!isset($obj->NewDataSet->Table->$field)) {
                return null;
            }
            // @codingStandardsIgnoreEnd

            // @codingStandardsIgnoreStart
            $table = $obj->NewDataSet->Table;
            // @codingStandardsIgnoreEnd
        } catch (Exception $e) {
            return null;
        }
        
        return $table;
    }

    public function raw($wsdl, $method, $params)
    {
        try {
            $client = new SoapClient($wsdl);
            $response = $client->$method($params);

            return $response;
        } catch (Exception $e) {
            return null;
        }
    }

    public function createLabel($wsdl, $idOrdenEnvio, $nroEnvio)
    {
        try {

            $params = array(
                'idOrdenRetiro' => $idOrdenEnvio,
                'nroEnvio' => $nroEnvio
            );

            Mage::log($params, null, 'label.log');

            $result = $this->raw($wsdl, 'GetHtmlDeEtiquetasPorOrdenOrNumeroEnvio', $params);

            Mage::log($result, null, 'label.log');

            // @codingStandardsIgnoreStart
            if (empty($result) || empty($result->GetHtmlDeEtiquetasPorOrdenOrNumeroEnvioResult)) {
                return "";
            }
            // @codingStandardsIgnoreEnd

            // @codingStandardsIgnoreStart
            return $result->GetHtmlDeEtiquetasPorOrdenOrNumeroEnvioResult;
            // @codingStandardsIgnoreEnd
        } catch (Exception $ex) {
            Mage::logException($ex);
            throw $ex;
        }
    }

    public function generateLabel($orderRetiro, $numeroEnvio)
    {
        $wsdl = Mage::getStoreConfig('oca/ws/url');

        $method = 'GetHtmlDeEtiquetasLockersPorOrdenOrNumeroEnvio';

        $params = array(
            'idOrdenRetiro' => $orderRetiro,
            'nroEnvio' => $numeroEnvio
        );

        $result = $this->raw($wsdl, $method, $params);

        // @codingStandardsIgnoreStart
        if (empty($result->GetHtmlDeEtiquetasLockersPorOrdenOrNumeroEnvioResult)) {
            return '';
        }

        Mage::log($result, null, 'trace.log');

        // @codingStandardsIgnoreStart
        return $result->GetHtmlDeEtiquetasLockersPorOrdenOrNumeroEnvioResult;
        // @codingStandardsIgnoreEnd
    }

    public function exec_ingreso_or($wsdl, $method, $params, $field)
    {
        $client = new SoapClient($wsdl);
        $response = $client->$method($params);

        Mage::log($response, null, 'exec_ingreso_or.log', true);

        $table = null;
        
        try {
            $root = null;
            foreach (array_keys($response) as $key) {
                $root = $key;
                break;
            }
            
            if (!isset($response->$root->any)) {
                return null;
            }

            $xml = $response->$root->any; 
            $obj = simplexml_load_string($xml);

            // @codingStandardsIgnoreStart
            if (!isset($obj->Resultado->Resumen->$field)) {
                return null;
            }
            // @codingStandardsIgnoreEnd

            // @codingStandardsIgnoreStart
            if (isset($obj->Resultado->DetalleRechazos)) {
                if (isset($obj->Resultado->DetalleRechazos->Motivo)) {
                    $this->setError($obj->Resultado->DetalleRechazos->Motivo);
                }
                return null;
            }
            // @codingStandardsIgnoreEnd

            // @codingStandardsIgnoreStart
            $table = $obj->Resultado;
            // @codingStandardsIgnoreEnd
        } catch (Exception $e) {
            return null;
        }
        
        return $table;
    }
    
    public function getCentrosImposicionPorCP($codigoPostal)
    {
        $wsdl = Mage::getStoreConfig('oca/ws/url');

        $params = array(
            'CodigoPostal' => $codigoPostal
        );
        
        $method = 'GetCentrosImposicionPorCP';
        
        try {
            $client = new SoapClient($wsdl);
            $response = $client->$method($params); 

            $root = null;
            foreach (array_keys($response) as $key) {
                $root = $key;
                break;
            }

            if (!isset($response->$root->any)) {
                return null;
            }

            $xml = $response->$root->any; 
            $obj = simplexml_load_string($xml);

            // @codingStandardsIgnoreStart
            $tables = $obj->NewDataSet;
            // @codingStandardsIgnoreEnd

            return $tables;
        } catch (Exception $e) {
            Mage::logException($e);
        }
        return null;
    }
}

