<?php
/**
 * SemExpert_Oca
 *
 * @category    SemExpert
 * @package     SemExpert_Oca
 * @copyright   Copyright (c) SemExpert SRL (http://www.semexpert-ecommerce.com/)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class SemExpert_Oca_BoxesController extends Mage_Adminhtml_Controller_Action
{
    const HELPER_CLASS = 'oca';
    const BLOCK_PREFIX = 'oca/box';
    const CRUD_CLASS = 'oca/box';
    const ID_NAME = 'box_id';

    const SUCCESSFULLY_CREATED = 'Box was successfully created';
    const SUCCESSFULLY_UPDATED = 'Box was successfully updated';
    const SUCCESSFULLY_DELETED = 'Box successfully deleted';
    const CANNOT_DELETE = 'Default box cannot be deleted';
    const OBJECT_NOT_FOUND = "Box not found";

    const MAIN_GRID_URL = 'oca/boxes';

    public function indexAction()
    {
        $this->loadLayout()
            ->_addContent($this->getLayout()->createBlock(self::BLOCK_PREFIX . '_main'))
            ->renderLayout();
    }

    //specific for class
    public function makedefaultAction()
    {
        $objectId = $this->getRequest()->getParam(self::ID_NAME, false);
        if (empty($objectId)) {
            Mage::getSingleton('adminhtml/session')->addError($this->_getHelper()->__(self::OBJECT_NOT_FOUND));
            $this->_redirectReferer($this->getMainGridUrl());
            return;
        }

        try {
            $object = Mage::getModel(self::CRUD_CLASS)->load($objectId);
            if (!$object->getId()) {
                Mage::getSingleton('adminhtml/session')->addError($this->_getHelper()->__(self::OBJECT_NOT_FOUND));
                $this->_redirectReferer($this->getMainGridUrl());
                return;
            }

            $tableName = Mage::getModel('core/resource')->getTableName('oca/box');

            // seteo todos en false
            $connection = Mage::getSingleton('core/resource')->getConnection('core_write');
            $connection->update($tableName, array('is_default' => 0));

            // y luego este en true
            $object->setIsDefault(true)->save();
        } catch (Exception $e) {
            Mage::getSingleton('adminhtml/session')->addError($this->_getHelper()->__($e->getMessage()));
        }
        $this->getResponse()->setRedirect($this->getUrl('*/*/'));
    }

    public function newAction()
    {
        $this->loadLayout()
        ->_addContent($this->getLayout()->createBlock(self::BLOCK_PREFIX . '_new'))
        ->renderLayout();
    }

    public function postAction()
    {
        if ($data = $this->getRequest()->getPost()) {
            $object = Mage::getModel(self::CRUD_CLASS)->setData($data);
            try {
                //specific for this class
                if (!Mage::helper('oca/box')->defaultBoxExist()) {
                    $object->setIsDefault(true);
                }

                $object->save();
                
                Mage::getSingleton('adminhtml/session')->addSuccess(
                    $this->_getHelper()->__(self::SUCCESSFULLY_CREATED)
                );
            } catch (Exception $e){
                Mage::getSingleton('adminhtml/session')->addError($this->_getHelper()->__($e->getMessage()));
            }
        }

        $this->getResponse()->setRedirect($this->getUrl('*/*/'));
        return;
    }

    public function editAction()
    {
        $this->loadLayout();
        $this->_addContent($this->getLayout()->createBlock(self::BLOCK_PREFIX . '_edit'));
        $this->renderLayout();
    }

    public function saveAction()
    {
        $objectId = $this->getRequest()->getParam(self::ID_NAME, false);

        $data = $this->getRequest()->getPost();
        if ($data) {
            $object = Mage::getModel(self::CRUD_CLASS)->load($objectId);
            $object->addData($data);

            try {
                $object->setId($objectId)->save();

                Mage::getSingleton('adminhtml/session')->addSuccess(
                    $this->_getHelper()->__(self::SUCCESSFULLY_UPDATED)
                );

                // back to main grid
                $this->getResponse()->setRedirect($this->getMainGridUrl());
                return;
            } catch (Exception $e){
                Mage::getSingleton('adminhtml/session')->addError($this->_getHelper()->__($e->getMessage()));
            }
        }
        $this->_redirectReferer();
    }


    public function deleteAction()
    {
        $objectId = $this->getRequest()->getParam('id', false);

        if ($this->_checkCanDelete($objectId)) {
            try {
                Mage::getModel(self::CRUD_CLASS)->setId($objectId)->delete();
                Mage::getSingleton('adminhtml/session')->addSuccess(
                    $this->_getHelper()->__(self::SUCCESSFULLY_DELETED)
                );

                $this->getResponse()->setRedirect($this->getMainGridUrl());
                return;
            } catch (Exception $e){
                Mage::getSingleton('adminhtml/session')->addError($this->_getHelper()->__($e->getMessage()));
            }
        } else {
            Mage::getSingleton('adminhtml/session')->addError($this->_getHelper()->__(self::CANNOT_DELETE));
        }
        $this->_redirectReferer();
    }

    private function _checkCanDelete($objectId)
    {
        //specific for this class
        $object = Mage::getModel(self::CRUD_CLASS)->load($objectId);

        return !$object->getIsDefault();
    }

    protected function _getHelper()
    {
        return Mage::helper(self::HELPER_CLASS);
    }

    protected function getMainGridUrl()
    {
        return $this->getUrl(self::MAIN_GRID_URL);
    }
}
